@@ -1,5 +1,5 @@
-#ifndef AO_UVECTOR_11_H
-#define AO_UVECTOR_11_H
+#ifndef AOCOMMON_UVECTOR_11_H_
+#define AOCOMMON_UVECTOR_11_H_
 
 #include <algorithm>
 #include <cstring>
@@ -84,17 +84,17 @@ class UVector : private Alloc {
   /// Type of allocator used to allocate and deallocate space
   typedef Alloc allocator_type;
   /// Reference to element type
-  typedef Tp &reference;
+  typedef Tp& reference;
   /// Constant reference to element type
-  typedef const Tp &const_reference;
+  typedef const Tp& const_reference;
   /// Pointer to element type
-  typedef Tp *pointer;
+  typedef Tp* pointer;
   /// Pointer to constant element type
-  typedef const Tp *const_pointer;
+  typedef const Tp* const_pointer;
   /// Iterator type
-  typedef Tp *iterator;
+  typedef Tp* iterator;
   /// Iterator type of constant elements
-  typedef const Tp *const_iterator;
+  typedef const Tp* const_iterator;
   /// Reverse iterator type
   typedef std::reverse_iterator<iterator> reverse_iterator;
   /// Reverse iterator of constant elements
@@ -119,7 +119,7 @@ class UVector : private Alloc {
   /** @brief Construct an empty UVector.
    * @param allocator Allocator used for allocating and deallocating memory.
    */
-  explicit UVector(const allocator_type &allocator = Alloc()) noexcept
+  explicit UVector(const allocator_type& allocator = Alloc()) noexcept
       : Alloc(allocator),
         _begin(nullptr),
         _end(nullptr),
@@ -142,13 +142,13 @@ class UVector : private Alloc {
    * @param val Value to initialize all elements with
    * @param allocator Allocator used for allocating and deallocating memory.
    */
-  UVector(size_t n, const value_type &val,
-          const allocator_type &allocator = Alloc())
+  UVector(size_t n, const value_type& val,
+          const allocator_type& allocator = Alloc())
       : Alloc(allocator),
         _begin(allocate(n)),
         _end(_begin + n),
         _endOfStorage(_end) {
-    std::uninitialized_fill_n<Tp *, size_t>(_begin, n, val);
+    std::uninitialized_fill_n<Tp*, size_t>(_begin, n, val);
   }
 
   /** @brief Construct a vector by copying elements from a range.
@@ -158,7 +158,7 @@ class UVector : private Alloc {
    */
   template <class InputIterator>
   UVector(InputIterator first, InputIterator last,
-          const allocator_type &allocator = Alloc())
+          const allocator_type& allocator = Alloc())
       : Alloc(allocator) {
     construct_from_range<InputIterator>(first, last,
                                         std::is_integral<InputIterator>());
@@ -170,7 +170,7 @@ class UVector : private Alloc {
    * std::allocator_traits<Alloc>::select_on_container_copy_construction(other).
    * @param other Source UVector to be copied from.
    */
-  UVector(const UVector<Tp, Alloc> &other)
+  UVector(const UVector<Tp, Alloc>& other)
       : Alloc(
             std::allocator_traits<Alloc>::select_on_container_copy_construction(
                 static_cast<allocator_type>(other))),
@@ -184,7 +184,7 @@ class UVector : private Alloc {
    * @param other Source UVector to be copied from.
    * @param allocator Allocator used for allocating and deallocating memory.
    */
-  UVector(const UVector<Tp, Alloc> &other, const allocator_type &allocator)
+  UVector(const UVector<Tp, Alloc>& other, const allocator_type& allocator)
       : Alloc(allocator),
         _begin(allocate(other.size())),
         _end(_begin + other.size()),
@@ -195,7 +195,7 @@ class UVector : private Alloc {
   /** @brief Move construct a UVector.
    * @param other Source UVector to be moved from.
    */
-  UVector(UVector<Tp, Alloc> &&other) noexcept
+  UVector(UVector<Tp, Alloc>&& other) noexcept
       : Alloc(std::move(other)),
         _begin(other._begin),
         _end(other._end),
@@ -209,7 +209,7 @@ class UVector : private Alloc {
    * @param other Source UVector to be moved from.
    * @param allocator Allocator used for allocating and deallocating memory.
    */
-  UVector(UVector<Tp, Alloc> &&other, const allocator_type &allocator) noexcept
+  UVector(UVector<Tp, Alloc>&& other, const allocator_type& allocator) noexcept
       : Alloc(allocator),
         _begin(other._begin),
         _end(other._end),
@@ -224,7 +224,7 @@ class UVector : private Alloc {
    * @param allocator Allocator used for allocating and deallocating memory.
    */
   UVector(std::initializer_list<Tp> initlist,
-          const allocator_type &allocator = Alloc())
+          const allocator_type& allocator = Alloc())
       : Alloc(allocator),
         _begin(allocate(initlist.size())),
         _end(_begin + initlist.size()),
@@ -246,7 +246,7 @@ class UVector : private Alloc {
    * std::allocator_traits<Alloc>::propagate_on_container_copy_assignment() is
    * of true_type.
    */
-  UVector &operator=(const UVector<Tp, Alloc> &other) {
+  UVector& operator=(const UVector<Tp, Alloc>& other) {
     return assign_copy_from(
         other, typename std::allocator_traits<
                    Alloc>::propagate_on_container_copy_assignment());
@@ -257,7 +257,7 @@ class UVector : private Alloc {
    * std::allocator_traits<Alloc>::propagate_on_container_move_assignment() is
    * of true_type.
    */
-  UVector &operator=(UVector<Tp, Alloc> &&other) noexcept(
+  UVector& operator=(UVector<Tp, Alloc>&& other) noexcept(
       std::allocator_traits<
           Alloc>::propagate_on_container_move_assignment::value ||
       allocator_is_always_equal::value) {
@@ -346,7 +346,7 @@ class UVector : private Alloc {
    * @param n The new size of the container.
    * @param val New value of elements that get added to the container.
    */
-  void resize(size_t n, const Tp &val) {
+  void resize(size_t n, const Tp& val) {
     size_t oldSize = size();
     if (capacity() < n) {
       pointer newStorage = allocate(n);
@@ -356,8 +356,7 @@ class UVector : private Alloc {
       _endOfStorage = _begin + n;
     }
     _end = _begin + n;
-    if (oldSize < n)
-      std::uninitialized_fill<Tp *, size_t>(_begin + oldSize, _end, val);
+    if (oldSize < n) std::uninitialized_fill<Tp*>(_begin + oldSize, _end, val);
   }
 
   /** @brief Get the number of elements the container can currently hold without
@@ -418,16 +417,16 @@ class UVector : private Alloc {
   }
 
   /** @brief Get a reference to the element at the given index. */
-  Tp &operator[](size_t index) noexcept { return _begin[index]; }
+  Tp& operator[](size_t index) noexcept { return _begin[index]; }
 
   /** @brief Get a constant reference to the element at the given index. */
-  const Tp &operator[](size_t index) const noexcept { return _begin[index]; }
+  const Tp& operator[](size_t index) const noexcept { return _begin[index]; }
 
   /** @brief Get a reference to the element at the given index with bounds
    * checking.
    * @throws std::out_of_range when given index is past the last element.
    */
-  Tp &at(size_t index) {
+  Tp& at(size_t index) {
     check_bounds(index);
     return _begin[index];
   }
@@ -436,28 +435,28 @@ class UVector : private Alloc {
    * bounds checking.
    * @throws std::out_of_range when given index is past the last element.
    */
-  const Tp &at(size_t index) const {
+  const Tp& at(size_t index) const {
     check_bounds(index);
     return _begin[index];
   }
 
   /** @brief Get reference to first element in container. */
-  Tp &front() noexcept { return *_begin; }
+  Tp& front() noexcept { return *_begin; }
 
   /** @brief Get constant reference to first element in container. */
-  const Tp &front() const noexcept { return *_begin; }
+  const Tp& front() const noexcept { return *_begin; }
 
   /** @brief Get reference to last element in container. */
-  Tp &back() noexcept { return *(_end - 1); }
+  Tp& back() noexcept { return *(_end - 1); }
 
   /** @brief Get constant reference to last element in container. */
-  const Tp &back() const noexcept { return *(_end - 1); }
+  const Tp& back() const noexcept { return *(_end - 1); }
 
   /** @brief Get pointer to internal storage. */
-  Tp *data() noexcept { return _begin; }
+  Tp* data() noexcept { return _begin; }
 
   /** @brief Get constant pointer to internal storage. */
-  const Tp *data() const noexcept { return _begin; }
+  const Tp* data() const noexcept { return _begin; }
 
   /** @brief Assign this container to be equal to the given range.
    * @details The container will be resized to fit the length of the given
@@ -476,7 +475,7 @@ class UVector : private Alloc {
    * @param n New size of container
    * @param val Value to be assigned to all elements.
    */
-  void assign(size_t n, const Tp &val) {
+  void assign(size_t n, const Tp& val) {
     if (n > capacity()) {
       iterator newStorage = allocate(n);
       deallocate();
@@ -484,7 +483,7 @@ class UVector : private Alloc {
       _endOfStorage = _begin + n;
     }
     _end = _begin + n;
-    std::uninitialized_fill_n<Tp *, size_t>(_begin, n, val);
+    std::uninitialized_fill_n<Tp*, size_t>(_begin, n, val);
   }
 
   /** @brief Assign this container to an initializer list.
@@ -513,7 +512,7 @@ class UVector : private Alloc {
    * @details Iterators are invalidated.
    * @param item Value of new element.
    */
-  void push_back(const Tp &item) {
+  void push_back(const Tp& item) {
     if (_end == _endOfStorage) enlarge(enlarge_size(1));
     *_end = item;
     ++_end;
@@ -527,7 +526,7 @@ class UVector : private Alloc {
    * in over copying it in with @ref push_back(const Tp&).
    * @param item Value of new element.
    */
-  void push_back(Tp &&item) {
+  void push_back(Tp&& item) {
     if (_end == _endOfStorage) enlarge(enlarge_size(1));
     *_end = std::move(item);
     ++_end;
@@ -544,7 +543,7 @@ class UVector : private Alloc {
    * @param item Value of the new item.
    * @return Position of the new element.
    */
-  iterator insert(const_iterator position, const Tp &item) {
+  iterator insert(const_iterator position, const Tp& item) {
     if (_end == _endOfStorage) {
       size_t index = position - _begin;
       enlarge_for_insert(enlarge_size(1), index, 1);
@@ -567,7 +566,7 @@ class UVector : private Alloc {
    * @param val Value of the new item.
    * @return Position of the first new element.
    */
-  iterator insert(const_iterator position, size_t n, const Tp &val) {
+  iterator insert(const_iterator position, size_t n, const Tp& val) {
     if (capacity() < size() + n) {
       size_t index = position - _begin;
       enlarge_for_insert(enlarge_size(n), index, n);
@@ -576,8 +575,8 @@ class UVector : private Alloc {
       std::move_backward(const_cast<iterator>(position), _end, _end + n);
       _end += n;
     }
-    std::uninitialized_fill_n<Tp *, size_t>(const_cast<iterator>(position), n,
-                                            val);
+    std::uninitialized_fill_n<Tp*, size_t>(const_cast<iterator>(position), n,
+                                           val);
     return const_cast<iterator>(position);
   }
 
@@ -611,7 +610,7 @@ class UVector : private Alloc {
    * @param item Value of the new item.
    * @return Position of the new element.
    */
-  iterator insert(const_iterator position, Tp &&item) {
+  iterator insert(const_iterator position, Tp&& item) {
     if (_end == _endOfStorage) {
       size_t index = position - _begin;
       enlarge_for_insert(enlarge_size(1), index, 1);
@@ -690,7 +689,7 @@ class UVector : private Alloc {
    * @c propagate_on_container_swap is false.
    * @param other Other UVector whose contents it to be swapped with this.
    */
-  void swap(UVector<Tp, Alloc> &other) noexcept {
+  void swap(UVector<Tp, Alloc>& other) noexcept {
     swap(other,
          typename std::allocator_traits<Alloc>::propagate_on_container_swap());
   }
@@ -707,7 +706,7 @@ class UVector : private Alloc {
    * @return Position of the new element.
    */
   template <typename... Args>
-  iterator emplace(const_iterator position, Args &&... args) {
+  iterator emplace(const_iterator position, Args&&... args) {
     if (_end == _endOfStorage) {
       size_t index = position - _begin;
       enlarge_for_insert(enlarge_size(1), index, 1);
@@ -726,7 +725,7 @@ class UVector : private Alloc {
    * @param args List of arguments to be forwarded to construct the new element.
    */
   template <typename... Args>
-  void emplace_back(Args &&... args) {
+  void emplace_back(Args&&... args) {
     if (_end == _endOfStorage) enlarge(enlarge_size(1));
     *_end = Tp(std::forward<Args>(args)...);
     ++_end;
@@ -781,11 +780,11 @@ class UVector : private Alloc {
    * @param n Number of elements to add.
    * @param val Value of the new items.
    */
-  void push_back(size_t n, const Tp &val) {
+  void push_back(size_t n, const Tp& val) {
     if (capacity() - size() < n) {
       enlarge(enlarge_size(n));
     }
-    std::uninitialized_fill_n<Tp *, size_t>(_end, n, val);
+    std::uninitialized_fill_n<Tp*, size_t>(_end, n, val);
     _end += n;
   }
 
@@ -836,7 +835,7 @@ class UVector : private Alloc {
     _begin = allocate(n);
     _end = _begin + n;
     _endOfStorage = _end;
-    std::uninitialized_fill_n<Tp *, size_t>(_begin, n, val);
+    std::uninitialized_fill_n<Tp*, size_t>(_begin, n, val);
   }
 
   template <typename InputIterator>
@@ -846,7 +845,7 @@ class UVector : private Alloc {
     _begin = allocate(n);
     _end = _begin + n;
     _endOfStorage = _begin + n;
-    Tp *destIter = _begin;
+    Tp* destIter = _begin;
     while (first != last) {
       *destIter = *first;
       ++destIter;
@@ -874,7 +873,7 @@ class UVector : private Alloc {
       _endOfStorage = _begin + n;
     }
     _end = _begin + n;
-    std::uninitialized_fill_n<Tp *, size_t>(_begin, n, val);
+    std::uninitialized_fill_n<Tp*, size_t>(_begin, n, val);
   }
 
   template <typename InputIterator>
@@ -888,7 +887,7 @@ class UVector : private Alloc {
       _endOfStorage = _begin + n;
     }
     _end = _begin + n;
-    Tp *destIter = _begin;
+    Tp* destIter = _begin;
     while (first != last) {
       *destIter = *first;
       ++destIter;
@@ -915,8 +914,8 @@ class UVector : private Alloc {
       std::move_backward(const_cast<iterator>(position), _end, _end + n);
       _end += n;
     }
-    std::uninitialized_fill_n<Tp *, size_t>(const_cast<iterator>(position), n,
-                                            val);
+    std::uninitialized_fill_n<Tp*, size_t>(const_cast<iterator>(position), n,
+                                           val);
     return const_cast<iterator>(position);
   }
 
@@ -932,7 +931,7 @@ class UVector : private Alloc {
       std::move_backward(const_cast<iterator>(position), _end, _end + n);
       _end += n;
     }
-    Tp *destIter = const_cast<iterator>(position);
+    Tp* destIter = const_cast<iterator>(position);
     while (first != last) {
       *destIter = *first;
       ++destIter;
@@ -973,7 +972,7 @@ class UVector : private Alloc {
 
   // implementation of operator=(const&) without
   // propagate_on_container_copy_assignment
-  UVector &assign_copy_from(const UVector<Tp, Alloc> &other, std::false_type) {
+  UVector& assign_copy_from(const UVector<Tp, Alloc>& other, std::false_type) {
     const size_t n = other.size();
     if (n > capacity()) {
       iterator newStorage = allocate(n);
@@ -988,30 +987,30 @@ class UVector : private Alloc {
 
   // implementation of operator=(const&) with
   // propagate_on_container_copy_assignment
-  UVector &assign_copy_from(const UVector<Tp, Alloc> &other, std::true_type) {
+  UVector& assign_copy_from(const UVector<Tp, Alloc>& other, std::true_type) {
     if (allocator_is_always_equal() ||
-        static_cast<Alloc &>(other) == static_cast<Alloc &>(*this)) {
+        static_cast<Alloc&>(other) == static_cast<Alloc&>(*this)) {
       assign_copy_from(other, std::false_type());
     } else {
       const size_t n = other.size();
-      iterator newStorage = static_cast<Alloc &>(other).allocate(n);
+      iterator newStorage = static_cast<Alloc&>(other).allocate(n);
       deallocate();
       _begin = newStorage;
       _end = _begin + n;
       _endOfStorage = _end;
       std::copy(other._begin, other._begin + n, _begin);
-      Alloc::operator=(static_cast<Alloc &>(other));
+      Alloc::operator=(static_cast<Alloc&>(other));
     }
     return *this;
   }
 
   // implementation of operator=() without
   // propagate_on_container_move_assignment
-  UVector &assign_move_from(
-      UVector<Tp, Alloc> &&other,
+  UVector& assign_move_from(
+      UVector<Tp, Alloc>&& other,
       std::false_type) noexcept(allocator_is_always_equal::value) {
     if (allocator_is_always_equal::value ||
-        static_cast<Alloc &>(other) == static_cast<Alloc &>(*this)) {
+        static_cast<Alloc&>(other) == static_cast<Alloc&>(*this)) {
       deallocate();
       _begin = other._begin;
       _end = other._end;
@@ -1030,10 +1029,10 @@ class UVector : private Alloc {
   }
 
   // implementation of operator=() with propagate_on_container_move_assignment
-  UVector &assign_move_from(UVector<Tp, Alloc> &&other,
+  UVector& assign_move_from(UVector<Tp, Alloc>&& other,
                             std::true_type) noexcept {
     deallocate();
-    Alloc::operator=(std::move(static_cast<Alloc &>(other)));
+    Alloc::operator=(std::move(static_cast<Alloc&>(other)));
     _begin = other._begin;
     _end = other._end;
     _endOfStorage = other._endOfStorage;
@@ -1044,15 +1043,15 @@ class UVector : private Alloc {
   }
 
   // implementation of swap with propagate_on_container_swap
-  void swap(UVector<Tp, Alloc> &other, std::true_type) noexcept {
+  void swap(UVector<Tp, Alloc>& other, std::true_type) noexcept {
     std::swap(_begin, other._begin);
     std::swap(_end, other._end);
     std::swap(_endOfStorage, other._endOfStorage);
-    std::swap(static_cast<Alloc &>(other), static_cast<Alloc &>(*this));
+    std::swap(static_cast<Alloc&>(other), static_cast<Alloc&>(*this));
   }
 
   // implementation of swap without propagate_on_container_swap
-  void swap(UVector<Tp, Alloc> &other, std::false_type) noexcept {
+  void swap(UVector<Tp, Alloc>& other, std::false_type) noexcept {
     std::swap(_begin, other._begin);
     std::swap(_end, other._end);
     std::swap(_endOfStorage, other._endOfStorage);
@@ -1097,7 +1096,7 @@ class UVector : private Alloc {
     if (capacity() - size() < size_t(n)) {
       enlarge(enlarge_size(n));
     }
-    std::uninitialized_fill_n<Tp *, size_t>(_end, n, val);
+    std::uninitialized_fill_n<Tp*, size_t>(_end, n, val);
     _end += n;
   }
 
@@ -1118,16 +1117,16 @@ class UVector : private Alloc {
 
 /** @brief Compare two UVectors for equality. */
 template <class Tp, class Alloc>
-inline bool operator==(const UVector<Tp, Alloc> &lhs,
-                       const UVector<Tp, Alloc> &rhs) noexcept {
+inline bool operator==(const UVector<Tp, Alloc>& lhs,
+                       const UVector<Tp, Alloc>& rhs) noexcept {
   return lhs.size() == rhs.size() &&
          std::equal(lhs.begin(), lhs.end(), rhs.begin());
 }
 
 /** @brief Compare two UVectors for inequality. */
 template <class Tp, class Alloc>
-inline bool operator!=(const UVector<Tp, Alloc> &lhs,
-                       const UVector<Tp, Alloc> &rhs) noexcept {
+inline bool operator!=(const UVector<Tp, Alloc>& lhs,
+                       const UVector<Tp, Alloc>& rhs) noexcept {
   return !(lhs == rhs);
 }
 
@@ -1136,8 +1135,8 @@ inline bool operator!=(const UVector<Tp, Alloc> &lhs,
  * with the smallest size is consider to be smaller.
  */
 template <class Tp, class Alloc>
-inline bool operator<(const UVector<Tp, Alloc> &lhs,
-                      const UVector<Tp, Alloc> &rhs) noexcept {
+inline bool operator<(const UVector<Tp, Alloc>& lhs,
+                      const UVector<Tp, Alloc>& rhs) noexcept {
   const size_t minSize = std::min(lhs.size(), rhs.size());
   for (size_t i = 0; i != minSize; ++i) {
     if (lhs[i] < rhs[i])
@@ -1153,8 +1152,8 @@ inline bool operator<(const UVector<Tp, Alloc> &lhs,
  * with the smallest size is consider to be smaller.
  */
 template <class Tp, class Alloc>
-inline bool operator<=(const UVector<Tp, Alloc> &lhs,
-                       const UVector<Tp, Alloc> &rhs) noexcept {
+inline bool operator<=(const UVector<Tp, Alloc>& lhs,
+                       const UVector<Tp, Alloc>& rhs) noexcept {
   const size_t minSize = std::min(lhs.size(), rhs.size());
   for (size_t i = 0; i != minSize; ++i) {
     if (lhs[i] < rhs[i])
@@ -1170,8 +1169,8 @@ inline bool operator<=(const UVector<Tp, Alloc> &lhs,
  * with the smallest size is consider to be smaller.
  */
 template <class Tp, class Alloc>
-inline bool operator>(const UVector<Tp, Alloc> &lhs,
-                      const UVector<Tp, Alloc> &rhs) noexcept {
+inline bool operator>(const UVector<Tp, Alloc>& lhs,
+                      const UVector<Tp, Alloc>& rhs) noexcept {
   return rhs < lhs;
 }
 
@@ -1180,8 +1179,8 @@ inline bool operator>(const UVector<Tp, Alloc> &lhs,
  * with the smallest size is consider to be smaller.
  */
 template <class Tp, class Alloc>
-inline bool operator>=(const UVector<Tp, Alloc> &lhs,
-                       const UVector<Tp, Alloc> &rhs) noexcept {
+inline bool operator>=(const UVector<Tp, Alloc>& lhs,
+                       const UVector<Tp, Alloc>& rhs) noexcept {
   return rhs <= lhs;
 }
 
@@ -1196,7 +1195,7 @@ inline bool operator>=(const UVector<Tp, Alloc> &lhs,
  * @c propagate_on_container_swap is false.
  */
 template <class Tp, class Alloc>
-inline void swap(UVector<Tp, Alloc> &x, UVector<Tp, Alloc> &y) {
+inline void swap(UVector<Tp, Alloc>& x, UVector<Tp, Alloc>& y) {
   x.swap(y);
 }
 
