@@ -10,7 +10,8 @@
 #include <aocommon/image.h>
 #include <aocommon/system.h>
 
-#include <schaapcommon/fft/convolution.h>
+#include <schaapcommon/math/ellipse.h>
+#include <schaapcommon/math/convolution.h>
 #include <schaapcommon/fitters/gaussianfitter.h>
 
 #include "image_set.h"
@@ -42,47 +43,40 @@ void IuwtDeconvolutionAlgorithm::MeasureRMSPerScale(
     const float* image, const float* convolved_image, float* scratch,
     size_t end_scale, std::vector<ScaleResponse>& psf_response) {
   IuwtDecomposition imageIUWT(end_scale, width_, height_);
-  imageIUWT.Decompose(*static_for_, image, scratch, false);
+  imageIUWT.Decompose(image, scratch, false);
+
+  schaapcommon::math::Ellipse ellipse =
+      schaapcommon::fitters::Fit2DGaussianCentred(image, width_, height_, 2.0);
 
-  psf_major_ = 2.0;
-  psf_minor_ = 2.0;
-  psf_pa_ = 0.0;
   double fl = 0.0;
-  schaapcommon::fitters::GaussianFitter fitter;
-  fitter.Fit2DGaussianCentred(image, width_, height_, 2.0, psf_major_,
-                              psf_minor_, psf_pa_);
-
   double v = 1.0, x = width_ / 2, y = height_ / 2;
-  double bMaj = psf_major_, bMin = psf_minor_, bPA = psf_pa_;
-  fitter.Fit2DGaussianFull(image, width_, height_, v, x, y, bMaj, bMin, bPA,
-                           &fl);
+  schaapcommon::fitters::Fit2DGaussianFull(image, width_, height_, v, x, y,
+                                           ellipse.major, ellipse.minor,
+                                           ellipse.position_angle, &fl);
 
   psf_response.resize(end_scale);
   for (size_t scale = 0; scale != end_scale; ++scale) {
     psf_response[scale].rms = imageIUWT[scale].Coefficients().RMS();
     psf_response[scale].peak_response =
         CentralPeak(imageIUWT[scale].Coefficients());
-    bMaj = 2.0;
-    bMin = 2.0;
-    bPA = 0.0;
     v = 1.0;
     x = width_ / 2;
     y = height_ / 2;
-    fitter.Fit2DGaussianFull(imageIUWT[scale].Coefficients().Data(), width_,
-                             height_, v, x, y, bMaj, bMin, bPA, &fl);
-    psf_response[scale].b_major = bMaj;
-    psf_response[scale].b_minor = bMin;
-    psf_response[scale].b_pa = bPA;
+    schaapcommon::fitters::Fit2DGaussianFull(
+        imageIUWT[scale].Coefficients().Data(), width_, height_, v, x, y,
+        ellipse.major, ellipse.minor, ellipse.position_angle, &fl);
+    psf_response[scale].b_major = ellipse.major;
+    psf_response[scale].b_minor = ellipse.minor;
+    psf_response[scale].b_pa = ellipse.position_angle;
   }
 
-  imageIUWT.Decompose(*static_for_, imageIUWT[1].Coefficients().Data(), scratch,
-                      false);
+  imageIUWT.Decompose(imageIUWT[1].Coefficients().Data(), scratch, false);
   for (size_t scale = 0; scale != end_scale; ++scale) {
     psf_response[scale].peak_response_to_next_scale =
         CentralPeak(imageIUWT[scale].Coefficients());
   }
 
-  imageIUWT.Decompose(*static_for_, convolved_image, scratch, false);
+  imageIUWT.Decompose(convolved_image, scratch, false);
 
   for (size_t scale = 0; scale != end_scale; ++scale) {
     psf_response[scale].convolvedPeakResponse =
@@ -341,11 +335,11 @@ bool IuwtDeconvolutionAlgorithm::RunConjugateGradient(
   for (size_t minorIter = 0; minorIter != 20; ++minorIter) {
     // scratch = gradient (x) psf
     scratch = gradient;
-    schaapcommon::fft::Convolve(scratch.Data(), psf_kernel.Data(), width,
-                                height, static_for_->NThreads());
+    schaapcommon::math::Convolve(scratch.Data(), psf_kernel.Data(), width,
+                                 height);
 
     // calc: IUWT gradient (x) psf
-    iuwt.Decompose(*static_for_, scratch.Data(), scratch.Data(), false);
+    iuwt.Decompose(scratch.Data(), scratch.Data(), false);
 
     // calc: mask IUWT gradient (x) psf
     iuwt.ApplyMask(mask);
@@ -386,9 +380,9 @@ bool IuwtDeconvolutionAlgorithm::RunConjugateGradient(
 
     // scratch = mask IUWT PSF (x) model
     scratch = structure_model;
-    schaapcommon::fft::Convolve(scratch.Data(), psf_kernel.Data(), width,
-                                height, static_for_->NThreads());
-    iuwt.Decompose(*static_for_, scratch.Data(), scratch.Data(), false);
+    schaapcommon::math::Convolve(scratch.Data(), psf_kernel.Data(), width,
+                                 height);
+    iuwt.Decompose(scratch.Data(), scratch.Data(), false);
     iuwt.ApplyMask(mask);
 
     float previousSNR = modelSNR;
@@ -422,7 +416,7 @@ bool IuwtDeconvolutionAlgorithm::FindAndDeconvolveStructure(
     const Image& psf_kernel, const std::vector<Image>& psfs, Image& scratch,
     ImageSet& structure_model, size_t cur_end_scale, size_t cur_min_scale,
     std::vector<IuwtDeconvolutionAlgorithm::ValComponent>& max_components) {
-  iuwt.Decompose(*static_for_, dirty.Data(), scratch.Data(), false);
+  iuwt.Decompose(dirty.Data(), scratch.Data(), false);
   aocommon::UVector<float> thresholds(cur_end_scale);
   rmses_.resize(cur_end_scale);
   for (size_t scale = 0; scale != cur_end_scale; ++scale) {
@@ -539,9 +533,8 @@ bool IuwtDeconvolutionAlgorithm::FillAndDeconvolveStructure(
     TrimPsf(smallPSF, psf, newWidth, newHeight);
 
     Image smallPSFKernel(smallPSF.Width(), smallPSF.Height());
-    schaapcommon::fft::PrepareConvolutionKernel(
-        smallPSFKernel.Data(), smallPSF.Data(), newWidth, newHeight,
-        static_for_->NThreads());
+    schaapcommon::math::PrepareConvolutionKernel(
+        smallPSFKernel.Data(), smallPSF.Data(), newWidth, newHeight);
 
     scratch = Image(dirty.Width(), dirty.Height());
 
@@ -600,7 +593,7 @@ bool IuwtDeconvolutionAlgorithm::FillAndDeconvolveStructure(
     }
 
     // get undeconvolved dirty
-    iuwt.Decompose(*static_for_, dirty.Data(), scratch.Data(), false);
+    iuwt.Decompose(dirty.Data(), scratch.Data(), false);
 
     iuwt.ApplyMask(mask);
     iuwt.Recompose(scratch, false);
@@ -614,8 +607,8 @@ bool IuwtDeconvolutionAlgorithm::FillAndDeconvolveStructure(
 
     float rmsBefore = dirty.RMS();
     scratch = structureModel;
-    schaapcommon::fft::Convolve(scratch.Data(), psf_kernel.Data(), width,
-                                height, static_for_->NThreads());
+    schaapcommon::math::Convolve(scratch.Data(), psf_kernel.Data(), width,
+                                 height);
     maskedDirty = dirty;  // we use maskedDirty as temporary
     maskedDirty.AddWithFactor(scratch, -minor_loop_gain_);
     float rmsAfter = maskedDirty.RMS();
@@ -685,12 +678,12 @@ void IuwtDeconvolutionAlgorithm::PerformSubImageFitSingle(
   size_t width = iuwt.Width(), height = iuwt.Height();
 
   Image psfKernel(width, height);
-  schaapcommon::fft::PrepareConvolutionKernel(
-      psfKernel.Data(), psf.Data(), width, height, static_for_->NThreads());
+  schaapcommon::math::PrepareConvolutionKernel(psfKernel.Data(), psf.Data(),
+                                               width, height);
 
   Image& maskedDirty = scratch_b;
 
-  iuwt.Decompose(*static_for_, sub_dirty.Data(), sub_dirty.Data(), false);
+  iuwt.Decompose(sub_dirty.Data(), sub_dirty.Data(), false);
   iuwt.ApplyMask(mask);
   iuwt.Recompose(maskedDirty, false);
   aocommon::UVector<bool> mask2d(structure_model.Size(), false);
@@ -765,9 +758,8 @@ float IuwtDeconvolutionAlgorithm::PerformSubImageComponentFitBoxed(
     Image smallPsf;
     TrimPsf(smallPsf, psf, newWidth, newHeight);
     Image smallPsfKernel(smallPsf.Width(), smallPsf.Height());
-    schaapcommon::fft::PrepareConvolutionKernel(
-        smallPsfKernel.Data(), smallPsf.Data(), newWidth, newHeight,
-        static_for_->NThreads());
+    schaapcommon::math::PrepareConvolutionKernel(
+        smallPsfKernel.Data(), smallPsf.Data(), newWidth, newHeight);
 
     Image smallMaskedDirty;
     Trim(smallMaskedDirty, masked_dirty, x1, y1, x2, y2);
@@ -789,9 +781,8 @@ float IuwtDeconvolutionAlgorithm::PerformSubImageComponentFit(
     size_t y_offset) {
   const size_t width = iuwt.Width(), height = iuwt.Height();
   // Calculate IUWT^-1 mask IUWT model (x) PSF
-  schaapcommon::fft::Convolve(model.Data(), psfKernel.Data(), width, height,
-                              static_for_->NThreads());
-  iuwt.Decompose(*static_for_, model.Data(), model.Data(), false);
+  schaapcommon::math::Convolve(model.Data(), psfKernel.Data(), width, height);
+  iuwt.Decompose(model.Data(), model.Data(), false);
   iuwt.ApplyMask(mask);
   iuwt.Recompose(model, false);
 
@@ -813,9 +804,6 @@ float IuwtDeconvolutionAlgorithm::PerformMajorIteration(
     size_t& iter_counter, size_t n_iter, ImageSet& model_set,
     ImageSet& dirty_set, const std::vector<aocommon::Image>& psfs,
     bool& reached_major_threshold) {
-  aocommon::StaticFor<size_t> static_for(aocommon::system::ProcessorCount());
-  static_for_ = &static_for;
-
   reached_major_threshold = false;
   if (iter_counter == n_iter) return 0.0;
 
@@ -836,16 +824,16 @@ float IuwtDeconvolutionAlgorithm::PerformMajorIteration(
 
   // Prepare the PSF for convolutions later on
   Image psfKernel(width_, height_);
-  schaapcommon::fft::PrepareConvolutionKernel(
-      psfKernel.Data(), psf.Data(), width_, height_, static_for.NThreads());
+  schaapcommon::math::PrepareConvolutionKernel(psfKernel.Data(), psf.Data(),
+                                               width_, height_);
 
   std::cout << "Measuring PSF...\n";
   {
     Image convolvedPSF(psf);
     Image scratch(width_, height_);
 
-    schaapcommon::fft::Convolve(convolvedPSF.Data(), psfKernel.Data(), width_,
-                                height_, static_for.NThreads());
+    schaapcommon::math::Convolve(convolvedPSF.Data(), psfKernel.Data(), width_,
+                                 height_);
     MeasureRMSPerScale(psf.Data(), convolvedPSF.Data(), scratch.Data(),
                        maxScale, psf_response_);
   }
@@ -864,8 +852,8 @@ float IuwtDeconvolutionAlgorithm::PerformMajorIteration(
   do {
     std::cout << "*** Deconvolution iteration " << iter_counter << " ***\n";
     dirtyBeforeIteration = dirty;
-    schaapcommon::fft::PrepareConvolutionKernel(
-        psfKernel.Data(), psf.Data(), width_, height_, static_for.NThreads());
+    schaapcommon::math::PrepareConvolutionKernel(psfKernel.Data(), psf.Data(),
+                                                 width_, height_);
     std::vector<ValComponent> max_components;
     Image scratch(width_, height_);
     bool succeeded = FindAndDeconvolveStructure(
@@ -880,11 +868,10 @@ float IuwtDeconvolutionAlgorithm::PerformMajorIteration(
       for (size_t i = 0; i != dirty_set.Size(); ++i) {
         scratch = structureModel[i];
         size_t psfIndex = dirty_set.PsfIndex(i);
-        schaapcommon::fft::PrepareConvolutionKernel(
-            psfKernel.Data(), psfs[psfIndex].Data(), width_, height_,
-            static_for.NThreads());
-        schaapcommon::fft::Convolve(scratch.Data(), psfKernel.Data(), width_,
-                                    height_, static_for.NThreads());
+        schaapcommon::math::PrepareConvolutionKernel(
+            psfKernel.Data(), psfs[psfIndex].Data(), width_, height_);
+        schaapcommon::math::Convolve(scratch.Data(), psfKernel.Data(), width_,
+                                     height_);
         Subtract(dirty_set.Data(i), scratch);
       }
       dirty_set.GetLinearIntegrated(dirty);
