@@ -44,8 +44,8 @@ BOOST_AUTO_TEST_CASE(partialSubtractImageAVXPerformance) {
   NoiseFixture<double> f;
   size_t x = f.n / 2, y = f.n / 2;
   for (size_t repeat = 0; repeat != nRepeats; ++repeat)
-    SimpleClean::PartialSubtractImageAVX(f.img.data(), f.n, f.n, f.psf.data(),
-                                         f.n, f.n, x, y, 0.5, 0, f.n / 2);
+    simple_clean::PartialSubtractImageAVX(f.img.data(), f.n, f.n, f.psf.data(),
+                                          f.n, f.n, x, y, 0.5, 0, f.n / 2);
   BOOST_CHECK(true);
 }
 #endif
