@@ -2,9 +2,9 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "soltab.h"
-#include "gridinterpolate.h"
 
 #include <algorithm>
+#include <array>
 #include <cassert>
 #include <cstdlib>
 #include <cmath>
@@ -15,6 +15,8 @@
 
 #include <hdf5.h>
 
+#include "gridinterpolate.h"
+
 // using namespace std;
 namespace schaapcommon {
 namespace h5parm {
@@ -37,6 +39,51 @@ std::vector<std::string> Tokenize(const std::string& str,
 }
 }  // namespace
 
+std::vector<double> SelectAxisValues(std::vector<double>&& axis,
+                                     double requested_start,
+                                     double requested_end, bool nearest,
+                                     size_t& start_index) {
+  assert(!axis.empty());
+  assert(std::is_sorted(axis.cbegin(), axis.cend()));
+  assert(requested_start <= requested_end);
+  //  ReadAxes() already checked that the time axis is sorted.
+  std::vector<double>::const_iterator lower =
+      std::lower_bound(axis.cbegin(), axis.cend(), requested_start);
+  std::vector<double>::const_iterator upper;
+  if (nearest) {
+    upper = std::upper_bound(lower, axis.cend(), requested_end);
+    // move upper to nearest element
+    if (upper > axis.cbegin() && upper != axis.cend()) {
+      if (requested_end - *(upper - 1) < *upper - requested_end) {
+        --upper;
+      }
+    }
+    // move lower to the nearest element
+    if (lower > axis.cbegin()) {
+      if (lower == axis.cend() ||
+          requested_start - *(lower - 1) < *lower - requested_start) {
+        --lower;
+      }
+    }
+  } else {
+    upper = std::lower_bound(lower, axis.cend(), requested_end);
+    // lower may be larger than the requested value; if this case move lower
+    // one value back.
+    if (lower > axis.cbegin() &&
+        (lower == axis.cend() || *lower > requested_start)) {
+      --lower;
+    }
+  }
+  // upper must point to the element after the element that is nearest/bounded:
+  if (upper != axis.cend()) {
+    ++upper;
+  }
+  start_index = lower - axis.cbegin();
+  axis.assign(lower, upper);
+  assert(!axis.empty());
+  return std::move(axis);
+}
+
 SolTab::SolTab(H5::Group group, const std::string& type,
                const std::vector<AxisInfo>& axes)
     : H5::Group(group), type_(type), axes_(axes) {
@@ -48,12 +95,18 @@ SolTab::SolTab(H5::Group group, const std::string& type,
 
 SolTab::SolTab(H5::Group& group) : H5::Group(group) {
   // Read the type from the "TITLE" attribute
+  if (!attrExists("TITLE")) {
+    throw std::runtime_error("H5 attribute TITLE not found in " + GetName());
+  }
   H5::Attribute typeattr = openAttribute("TITLE");
+  if (typeattr.getDataType().isVariableStr()) {
+    throw std::runtime_error(
+        "H5Parm attribute TITLE is a variable string, which is not supported");
+  }
   hsize_t typenamelen = typeattr.getDataType().getSize();
-  char typecstr[typenamelen + 1];
-  typecstr[typenamelen] = '\0';
-  typeattr.read(typeattr.getDataType(), &typecstr);
-  type_ = typecstr;
+  std::vector<char> type_chars(typenamelen + 1, '\0');
+  typeattr.read(typeattr.getDataType(), type_chars.data());
+  type_ = type_chars.data();
 
   ReadAxes();
 }
@@ -67,7 +120,7 @@ void SolTab::AddVersionStamp(H5::Group& node) {
   attr.write(H5::StrType(H5::PredType::C_S1, 3), "1.0");
 }
 
-AxisInfo SolTab::GetAxis(unsigned int i) const { return axes_[i]; }
+AxisInfo SolTab::GetAxis(size_t i) const { return axes_[i]; }
 
 AxisInfo SolTab::GetAxis(const std::string& axis_name) const {
   for (const AxisInfo& axis_info : axes_) {
@@ -101,7 +154,7 @@ void SolTab::SetValues(const std::vector<double>& vals,
   size_t expectedsize = 1;
   std::string axesstr = axes_.front().name;
   std::vector<hsize_t> dims(axes_.size());
-  for (unsigned int i = 0; i < axes_.size(); ++i) {
+  for (size_t i = 0; i < axes_.size(); ++i) {
     dims[i] = axes_[i].size;
     expectedsize *= dims[i];
     if (i > 0) {
@@ -188,19 +241,19 @@ void SolTab::SetValues(const std::vector<double>& vals,
   attr.write(H5::StrType(H5::PredType::C_S1, axesstr.size()), axesstr);
 }
 
-void SolTab::SetComplexValues(const std::vector<std::complex<double>>& vals,
+void SolTab::SetComplexValues(const std::vector<std::complex<double>>& values,
                               const std::vector<double>& weights,
                               bool to_amplitudes, const std::string& history) {
-  // Convert values to real numbers by taking amplitude or argument
-  std::vector<double> realvals(vals.size());
+  // Convert values to real numbers by taking amplitude or argument.
+  std::vector<double> real_values(values.size());
 
-  if (to_amplitudes) {
-    std::transform(vals.begin(), vals.end(), realvals.begin(), TakeAbs);
-  } else {  // Phase only
-    std::transform(vals.begin(), vals.end(), realvals.begin(), TakeArg);
-  }
+  std::transform(
+      values.begin(), values.end(), real_values.begin(),
+      to_amplitudes
+          ? [](std::complex<double> value) { return std::abs(value); }
+          : [](std::complex<double> value) { return std::arg(value); });
 
-  SetValues(realvals, weights, history);
+  SetValues(real_values, weights, history);
 }
 
 void SolTab::ReadAxes() {
@@ -220,12 +273,11 @@ void SolTab::ReadAxes() {
   }
 
   hsize_t axesstrlen = axesattr.getDataType().getSize();
-  char axescstr[axesstrlen + 1];
-  axescstr[axesstrlen] = '\0';
-  axesattr.read(axesattr.getDataType(), &axescstr);
-  std::vector<std::string> axesnames = Tokenize(axescstr, ",");
+  std::vector<char> axes_chars(axesstrlen + 1, '\0');
+  axesattr.read(axesattr.getDataType(), axes_chars.data());
+  std::vector<std::string> axesnames = Tokenize(axes_chars.data(), ",");
 
-  unsigned int ndims = axesnames.size();
+  const size_t ndims = axesnames.size();
 
   // Get number of dimensions and size of all dimensions
   H5::DataSpace ds = val.getSpace();
@@ -236,12 +288,11 @@ void SolTab::ReadAxes() {
         ") does not match number of axes in metadata (" +
         std::to_string(int(ndims)) + ")");
   }
-  hsize_t dims_out[ndims];
-  ds.getSimpleExtentDims(dims_out);
+  std::vector<hsize_t> dims_out(ndims, 0);
+  ds.getSimpleExtentDims(dims_out.data());
 
-  for (unsigned int i = 0; i < axesnames.size(); ++i) {
-    AxisInfo a{axesnames[i], static_cast<unsigned int>(dims_out[i])};
-    axes_.push_back(a);
+  for (size_t i = 0; i < axesnames.size(); ++i) {
+    axes_.push_back(AxisInfo{axesnames[i], dims_out[i]});
   }
 
   if (HasAxis("time")) {
@@ -253,85 +304,49 @@ void SolTab::ReadAxes() {
 }
 
 std::string SolTab::GetName() const {
-  size_t len = H5Iget_name(getId(), nullptr, 0);
-  char buffer[len];
-  H5Iget_name(getId(), buffer, len + 1);
+  if (!isValid(getId())) {
+    return "<invalid>";
+  }
+  const ssize_t len = H5Iget_name(getId(), nullptr, 0);
+  if (len < 0) {
+    throw std::runtime_error("Error retrieving H5 Group name.");
+  }
+  std::string name(len + 1, '\0');
+  H5Iget_name(getId(), name.data(), len + 1);
   // Strip leading /
-  return buffer + 1;
+  return name.data() + 1;
 }
 
-std::vector<double> SolTab::GetValuesOrWeights(const std::string& val_or_weight,
-                                               const std::string& ant_name,
-                                               const std::vector<double>& times,
-                                               const std::vector<double>& freqs,
-                                               unsigned int pol,
-                                               unsigned int dir, bool nearest) {
-  std::vector<double> res(times.size() * freqs.size());
+std::vector<double> SolTab::GetValues(const std::string& antenna_name,
+                                      const std::vector<double>& times,
+                                      const std::vector<double>& frequencies,
+                                      size_t polarization, size_t direction,
+                                      bool nearest) const {
+  std::vector<double> values =
+      GetValuesOrWeights("val", antenna_name, times, frequencies, polarization,
+                         direction, nearest);
 
-  unsigned int start_time_slot = 0;
-  unsigned int num_time_h5 = 1;
+  const std::vector<double> weights =
+      GetValuesOrWeights("weight", antenna_name, times, frequencies,
+                         polarization, direction, nearest);
 
-  assert(!times.empty());
-  assert(!freqs.empty());
-  unsigned int freq_start = 0;
-  unsigned int num_freq_h5 = 1;
+  ApplyFlags(values, weights);
 
-  std::vector<double> freq_axis_h5(1, 0.);
-  std::vector<double> time_axis_h5(1, 0.);
-  if (HasAxis("time")) {
-    time_axis_h5 = GetRealAxis("time");
-    if (times.size() > 1) {
-      num_time_h5 = time_axis_h5.size();
-    } else {
-      // When only one time is requested, find the nearest H5 time indices
-      // and only read H5 data for those indices.
+  return values;
+}
 
-      // ReadAxes() already checked that the "time" axis is sorted.
-      const auto lower = std::lower_bound(time_axis_h5.begin(),
-                                          time_axis_h5.end(), times.front());
-      if (lower == time_axis_h5.begin()) {
-        // start_time_slot remains 0
-        time_axis_h5.resize(1);  // Keep the first item only.
-      } else if (lower == time_axis_h5.end()) {
-        start_time_slot = time_axis_h5.size() - 1;
-        time_axis_h5 = {time_axis_h5.back()};
-      } else {
-        // The time lies between *(lower-1) and *lower.
-        start_time_slot = std::distance(time_axis_h5.begin(), lower) - 1;
-        if (nearest) {
-          // Only use the nearest entry.
-          if (times.front() - *(lower - 1) > *lower - times.front()) {
-            ++start_time_slot;
-          }
-          time_axis_h5 = {time_axis_h5[start_time_slot]};
-        } else {
-          // Only load the two entries around the entry.
-          num_time_h5 = 2;
-          time_axis_h5 = {*(lower - 1), *lower};
-        }
-      }
-    }
-  }
-  if (HasAxis("freq")) {
-    std::vector<double> full_freq_axis_h5 = GetRealAxis("freq");
-    freq_start = GetFreqIndex(freqs.front());
-    num_freq_h5 = GetFreqIndex(freqs.back()) - freq_start + 1;
-    freq_axis_h5 = std::vector<double>(
-        full_freq_axis_h5.begin() + freq_start,
-        full_freq_axis_h5.begin() + freq_start + num_freq_h5);
-  }
-  if (HasAxis("pol")) {
-    unsigned int num_pol_h5 = GetAxis("pol").size;
-    if (pol > num_pol_h5 - 1) {
-      throw std::runtime_error("Polarization " + std::to_string(pol) +
-                               " requested from H5Parm, but only " +
-                               std::to_string(num_pol_h5) +
-                               " polarizations are in there.");
-    }
-  }
-  const std::vector<double> h5values =
-      GetValuesOrWeights(val_or_weight, ant_name, start_time_slot, num_time_h5,
-                         1, freq_start, num_freq_h5, 1, pol, dir);
+std::vector<double> SolTab::GetValuesOrWeights(
+    const std::string& val_or_weight, const std::string& antenna_name,
+    const std::vector<double>& times, const std::vector<double>& frequencies,
+    size_t polarization, size_t direction, bool nearest) const {
+  TimesAndFrequencies times_and_frequencies = GetTimesAndFrequencies(
+      times, frequencies, polarization, direction, nearest);
+
+  const std::vector<double> sub_array = GetSubArray(
+      val_or_weight, antenna_name, times_and_frequencies.start_time_index,
+      times_and_frequencies.num_times, 1,
+      times_and_frequencies.start_freq_index, times_and_frequencies.num_freqs,
+      1, polarization, direction);
 
   MemoryLayout mem_layout = MemoryLayout::kRowMajor;
   // If the frequency index is lower than the time index, time will be the
@@ -341,59 +356,117 @@ std::vector<double> SolTab::GetValuesOrWeights(const std::string& val_or_weight,
       GetAxisIndex("freq") < GetAxisIndex("time")) {
     mem_layout = MemoryLayout::kColumnMajor;
   }
-  return GridNearestNeighbor(time_axis_h5, freq_axis_h5, times, freqs, h5values,
-                             mem_layout, nearest);
+
+  return GridNearestNeighbor(times_and_frequencies.time_axis,
+                             times_and_frequencies.freq_axis, times,
+                             frequencies, sub_array, mem_layout, nearest);
 }
 
-std::vector<double> SolTab::GetValuesOrWeights(
-    const std::string& val_or_weight, const std::string& ant_name,
-    unsigned int starttimeslot, unsigned int ntime, unsigned int timestep,
-    unsigned int startfreq, unsigned int nfreq, unsigned int freqstep,
-    unsigned int pol, unsigned int dir) {
-  std::vector<double> res(ntime * nfreq);
+void SolTab::ApplyFlags(std::vector<double>& values,
+                        const std::vector<double>& weights) {
+  assert(values.size() == weights.size());
+  for (size_t i = 0; i < values.size(); ++i) {
+    if (weights[i] == 0.0) {
+      values[i] = std::numeric_limits<double>::quiet_NaN();
+    }
+  }
+}
+
+TimesAndFrequencies SolTab::GetTimesAndFrequencies(
+    const std::vector<double>& times, const std::vector<double>& frequencies,
+    size_t pol, size_t dir, bool nearest) const {
+  assert(!times.empty());
+  assert(!frequencies.empty());
+
+  TimesAndFrequencies tf;
+  tf.time_axis = {0.0};
+  tf.start_time_index = 0;
+  tf.freq_axis = {0.0};
+  tf.start_freq_index = 0;
+  tf.num_freqs = 1;
+
+  if (HasAxis("time")) {
+    tf.time_axis = SelectAxisValues(GetRealAxis("time"), times.front(),
+                                    times.back(), nearest, tf.start_time_index);
+  }
+  tf.num_times = tf.time_axis.size();
+  if (HasAxis("freq")) {
+    std::vector<double> full_freq_axis_h5 = GetRealAxis("freq");
+    tf.start_freq_index = GetFreqIndex(frequencies.front());
+    tf.num_freqs = GetFreqIndex(frequencies.back()) - tf.start_freq_index + 1;
+    tf.freq_axis = std::vector<double>(
+        full_freq_axis_h5.begin() + tf.start_freq_index,
+        full_freq_axis_h5.begin() + tf.start_freq_index + tf.num_freqs);
+  }
+  if (HasAxis("pol")) {
+    const size_t num_pol_h5 = GetAxis("pol").size;
+    if (pol > num_pol_h5 - 1) {
+      throw std::runtime_error("Polarization " + std::to_string(pol) +
+                               " requested from H5Parm, but only " +
+                               std::to_string(num_pol_h5) +
+                               " polarizations are in there.");
+    }
+  }
+
+  return tf;
+}
+
+std::vector<double> SolTab::GetSubArray(
+    const std::string& val_or_weight, const std::string& antenna_name,
+    const size_t start_time_index, const size_t n_times, const size_t time_step,
+    const size_t start_freq_index, const size_t n_freqs, const size_t freq_step,
+    const size_t polarization, const size_t direction) const {
+  std::vector<double> res(n_times * n_freqs);
   H5::DataSet val = openDataSet(val_or_weight);
 
   // Set offsets and strides
-  hsize_t memdims[axes_.size()];
-  hsize_t offset[axes_.size()];
-  hsize_t count[axes_.size()];
-  hsize_t stride[axes_.size()];
+  std::vector<hsize_t> memdims;
+  std::vector<hsize_t> offsets;
+  std::vector<hsize_t> counts;
+  std::vector<hsize_t> strides;
+  memdims.reserve(axes_.size());
+  offsets.reserve(axes_.size());
+  counts.reserve(axes_.size());
+  strides.reserve(axes_.size());
 
-  for (unsigned int i = 0; i < axes_.size(); ++i) {
-    stride[i] = 1;
-    count[i] = 1;
-    memdims[i] = 1;
-    if (axes_[i].name == "time") {
-      offset[i] = starttimeslot;
-      stride[i] = timestep;
-      count[i] = ntime;
-      memdims[i] = ntime;
-    } else if (axes_[i].name == "freq") {
-      offset[i] = startfreq;
-      stride[i] = freqstep;
-      count[i] = nfreq;
-      memdims[i] = nfreq;
-    } else if (axes_[i].name == "ant") {
-      offset[i] = GetAntIndex(ant_name);
-    } else if (axes_[i].name == "dir") {
-      offset[i] = dir;
-    } else if (axes_[i].name == "pol") {
-      offset[i] = pol;
-    } else {
-      if (axes_[i].size != 1) {
-        throw std::runtime_error("Axis \"" + axes_[i].name +
-                                 "\" in H5Parm is not understood");
-      }
-      offset[i] = 0;
+  for (const AxisInfo& axis_info : axes_) {
+    hsize_t stride = 1;
+    hsize_t count = 1;
+    hsize_t memdim = 1;
+    hsize_t offset = 0;
+    if (axis_info.name == "time") {
+      offset = start_time_index;
+      stride = time_step;
+      count = n_times;
+      memdim = n_times;
+    } else if (axis_info.name == "freq") {
+      offset = start_freq_index;
+      stride = freq_step;
+      count = n_freqs;
+      memdim = n_freqs;
+    } else if (axis_info.name == "ant") {
+      offset = GetAntIndex(antenna_name);
+    } else if (axis_info.name == "dir") {
+      offset = direction;
+    } else if (axis_info.name == "pol") {
+      offset = polarization;
+    } else if (axis_info.size != 1) {
+      throw std::runtime_error("Axis \"" + axis_info.name +
+                               "\" in H5Parm is not understood");
     }
+    memdims.push_back(memdim);
+    offsets.push_back(offset);
+    counts.push_back(count);
+    strides.push_back(stride);
   }
 
   H5::DataSpace dataspace = val.getSpace();
 
-  dataspace.selectHyperslab(H5S_SELECT_SET, count, offset, stride);
+  dataspace.selectHyperslab(H5S_SELECT_SET, counts.data(), offsets.data(),
+                            strides.data());
 
   // Setup memory dataspace
-  H5::DataSpace memspace(axes_.size(), memdims);
+  H5::DataSpace memspace(axes_.size(), memdims.data());
   try {
     val.read(res.data(), H5::PredType::NATIVE_DOUBLE, memspace, dataspace);
   } catch (H5::DataSetIException& e) {
@@ -403,28 +476,73 @@ std::vector<double> SolTab::GetValuesOrWeights(
   return res;
 }
 
+std::vector<double> SolTab::GetCompleteArray(
+    const std::string& value_or_weight, const size_t n_axes,
+    const std::array<size_t, 5>& size_axis) const {
+  std::vector<hsize_t> memory_dimensions;
+  std::vector<hsize_t> offsets;
+  std::vector<hsize_t> counts;
+  std::vector<hsize_t> strides;
+  size_t result_size = 1;
+
+  for (size_t i = 0; i < n_axes; i++) {
+    memory_dimensions.push_back(size_axis[i]);
+    offsets.push_back(0);
+    counts.push_back(size_axis[i]);
+    strides.push_back(1);
+    result_size *= size_axis[i];
+  }
+
+  std::vector<double> result(result_size);
+  H5::DataSet val = openDataSet(value_or_weight);
+  H5::DataSpace dataspace = val.getSpace();
+  dataspace.selectHyperslab(H5S_SELECT_SET, counts.data(), offsets.data(),
+                            strides.data());
+  // Setup memory dataspace
+  H5::DataSpace memory_space(n_axes, memory_dimensions.data());
+  try {
+    val.read(result.data(), H5::PredType::NATIVE_DOUBLE, memory_space,
+             dataspace);
+  } catch (H5::DataSetIException& e) {
+    e.printErrorStack();
+    throw std::runtime_error("Could not read data");
+  }
+
+  return result;
+}
+
 void SolTab::SetAntennas(const std::vector<std::string>& sol_antennas) {
   // TODO: check that antenna is present in antenna table in solset
-  hsize_t dims[1];
-  dims[0] = sol_antennas.size();
+  std::array<hsize_t, 1> dimensions{{sol_antennas.size()}};
 
   size_t str_max_length = 1;
   for (const std::string& name : sol_antennas) {
     str_max_length = std::max(str_max_length, name.length());
   }
 
+  if (nameExists("ant")) unlink("ant");
+
   // Create dataset
-  H5::DataSpace dataspace(1, dims, nullptr);
-  H5::DataSet dataset = createDataSet(
-      "ant", H5::StrType(H5::PredType::C_S1, str_max_length), dataspace);
+  H5::DataSpace dataspace(dimensions.size(), dimensions.data(), nullptr);
+  H5::DataType datatype = H5::StrType(H5::PredType::C_S1, str_max_length);
+  H5::DataSet dataset = createDataSet("ant", datatype, dataspace);
 
   // Prepare data
-  char ant_array[sol_antennas.size()][str_max_length];
-  for (unsigned int i = 0; i < sol_antennas.size(); ++i) {
-    strncpy(ant_array[i], sol_antennas[i].c_str(), str_max_length);
+  std::vector<char> ant_array(sol_antennas.size() * str_max_length);
+  char* ant_array_position = ant_array.data();
+  for (const std::string& antenna_name : sol_antennas) {
+    strncpy(ant_array_position, antenna_name.c_str(), str_max_length);
+    ant_array_position += str_max_length;
   }
 
-  dataset.write(ant_array, H5::StrType(H5::PredType::C_S1, str_max_length));
+  dataset.write(ant_array.data(), datatype);
+
+  // Update cache
+  ant_list_ = sol_antennas;
+  ant_map_.clear();
+  for (size_t i = 0; i < sol_antennas.size(); ++i) {
+    ant_map_[sol_antennas[i]] = i;
+  }
 }
 
 void SolTab::SetAxisMeta(const std::string& meta_name, size_t num_char,
@@ -439,12 +557,14 @@ void SolTab::SetAxisMeta(const std::string& meta_name, size_t num_char,
 
   if (meta_vals.size() > 0) {
     // Prepare data
-    char src_array[meta_vals.size()][num_char];
-    for (unsigned int i = 0; i < meta_vals.size(); ++i) {
-      strncpy(src_array[i], meta_vals[i].c_str(), num_char);
+    std::vector<char> src_array(meta_vals.size() * num_char);
+    char* src_array_position = src_array.data();
+    for (const std::string& meta_value : meta_vals) {
+      strncpy(src_array_position, meta_value.c_str(), num_char);
+      src_array_position += num_char;
     }
 
-    dataset.write(src_array, H5::StrType(H5::PredType::C_S1, num_char));
+    dataset.write(src_array.data(), H5::StrType(H5::PredType::C_S1, num_char));
   }
 }
 
@@ -479,12 +599,13 @@ void SolTab::SetAxisMeta(const std::string& meta_name,
   }
 }
 
-hsize_t SolTab::GetAntIndex(const std::string& ant_name) {
-  return GetNamedIndex(ant_map_, "ant", ant_name);
-}
+void SolTab::FillCache(std::vector<std::string>& list,
+                       std::map<std::string, hsize_t>& map,
+                       const std::string& table_name) const {
+  if (!list.empty()) return;
+  assert(map.empty());
+  map.clear();  // Just in case.
 
-void SolTab::FillCache(std::map<std::string, hsize_t>& cache,
-                       const std::string& table_name) {
   H5::DataSet dataset;
   H5::DataSpace dataspace;
   try {
@@ -503,33 +624,49 @@ void SolTab::FillCache(std::map<std::string, hsize_t>& cache,
   // TODO: check that DataType is String
   hsize_t str_len = dataset.getDataType().getSize();
 
-  char el_names[str_len * dims[0]];
-  dataset.read(el_names, H5::StrType(H5::PredType::C_S1, str_len));
+  // Add 1 to the vector length, since the loop below modifies that element
+  // in its last iteration.
+  std::vector<char> el_names(str_len * dims[0] + 1, '\0');
+  dataset.read(el_names.data(), H5::StrType(H5::PredType::C_S1, str_len));
 
+  // Store the names in 'list' in their original order, for GetStringAxis.
+  // Also, map the names to their original index in 'map', for GetAntIndex and
+  // GetDirIndex.
+  list.reserve(dims[0]);
   for (hsize_t el_num = 0; el_num < dims[0]; ++el_num) {
-    char el_name_cstr[str_len + 1];
-    std::copy(el_names + el_num * str_len, el_names + (el_num + 1) * str_len,
-              el_name_cstr);
-    el_name_cstr[str_len] = '\0';
-    cache[el_name_cstr] = el_num;
+    const size_t name_index = el_num * str_len;
+    const size_t next_index = name_index + str_len;
+    const char saved = el_names[next_index];
+    el_names[next_index] = '\0';
+    list.emplace_back(&el_names[name_index]);
+    map[list.back()] = el_num;
+    el_names[next_index] = saved;
   }
 }
 
-hsize_t SolTab::GetNamedIndex(std::map<std::string, hsize_t>& cache,
+hsize_t SolTab::GetNamedIndex(std::vector<std::string>& cache_list,
+                              std::map<std::string, hsize_t>& cache_map,
                               const std::string& table_name,
-                              const std::string& element_name) {
-  // Initialize ant_map_ or dir_map_ on first use
-  if (cache.empty()) {
-    FillCache(cache, table_name);
-  }
-  auto it = cache.find(element_name);
-  if (it == cache.end()) {
+                              const std::string& element_name) const {
+  // Initialize ant_list_+ant_map_ or dir_list_+dir_map_ on first use.
+  FillCache(cache_list, cache_map, table_name);
+
+  auto it = cache_map.find(element_name);
+  if (it == cache_map.end()) {
     throw std::runtime_error("SolTab has no element " + element_name + " in " +
                              table_name);
   }
   return it->second;
 }
 
+hsize_t SolTab::GetAntIndex(const std::string& ant_name) const {
+  return GetNamedIndex(ant_list_, ant_map_, "ant", ant_name);
+}
+
+hsize_t SolTab::GetDirIndex(const std::string& direction_name) const {
+  return GetNamedIndex(dir_list_, dir_map_, "dir", direction_name);
+}
+
 hsize_t SolTab::GetFreqIndex(double freq) const {
   if (GetAxis("freq").size == 1) {
     return 0;
@@ -594,30 +731,21 @@ std::vector<double> SolTab::GetRealAxis(const std::string& axisname) const {
   return data;
 }
 
-std::vector<std::string> SolTab::GetStringAxis(const std::string& axis_name) {
-  std::map<std::string, hsize_t> cachemap;
-
+const std::vector<std::string>& SolTab::GetStringAxis(
+    const std::string& axis_name) const {
   if (axis_name == "dir") {
-    if (dir_map_.empty()) {
-      FillCache(dir_map_, "dir");
-    }
-    cachemap = dir_map_;
+    FillCache(dir_list_, dir_map_, "dir");
+    return dir_list_;
   } else if (axis_name == "ant") {
-    if (ant_map_.empty()) {
-      FillCache(ant_map_, "ant");
-    }
-    cachemap = ant_map_;
+    FillCache(ant_list_, ant_map_, "ant");
+    return ant_list_;
+  } else if (axis_name == "pol") {
+    FillCache(pol_list_, pol_map_, "pol");
+    return pol_list_;
   } else {
     throw std::runtime_error(
-        "Only string axes 'ant' and 'dir' supported for now.");
+        "Only string axes 'ant', 'dir' and 'pol' supported for now.");
   }
-
-  // Get the keys of the cache map and put them in a std::vector
-  std::vector<std::string> res;
-  for (const auto& key : cachemap) {
-    res.push_back(key.first);
-  }
-  return res;
 }
 
 hsize_t SolTab::GetTimeIndex(double time) const {
@@ -639,10 +767,6 @@ hsize_t SolTab::GetTimeIndex(double time) const {
   return 0;
 }
 
-hsize_t SolTab::GetDirIndex(const std::string& direction_name) {
-  return GetNamedIndex(dir_map_, "dir", direction_name);
-}
-
 double SolTab::GetInterval(const std::string& axis_name, size_t start) const {
   H5::DataSet dataset;
   H5::DataSpace dataspace;
