@@ -1,6 +1,8 @@
 #ifndef VISIBILITY_WEIGHTING_MODE_ENUM_H
 #define VISIBILITY_WEIGHTING_MODE_ENUM_H
 
+namespace wsclean {
+
 /**
  * This specifies several modi how the visibility weights
  * (normally stored in the SPECTRUM_WEIGHT column)
@@ -12,4 +14,6 @@ enum class VisibilityWeightingMode {
   UnitVisibilityWeighting
 };
 
-#endif
\ No newline at end of file
+}  // namespace wsclean
+
+#endif
