@@ -22,6 +22,8 @@
 #include <stack>
 #include <thread>
 
+namespace wsclean {
+
 /**
  * This class grids and/or samples visibilities to/from UV space.
  * It also executes the FFT(s) required and performs the w-term correction.
@@ -634,4 +636,6 @@ class WStackingGridder {
   size_t _nFFTThreads;
 };
 
+}  // namespace wsclean
+
 #endif
