@@ -7,6 +7,8 @@
 #include <memory>
 #include <vector>
 
+namespace wsclean {
+
 class CachedImageSet;
 
 class AverageBeam {
@@ -80,4 +82,6 @@ class AverageBeam {
   size_t matrix_height_;
 };
 
+}  // namespace wsclean
+
 #endif
