@@ -6,6 +6,8 @@
 
 #include "interface.h"
 
+namespace wsclean {
+
 class DummyGridder : public HighLevelGridderInterface {
  public:
   virtual ~DummyGridder() {}
@@ -60,4 +62,6 @@ class DummyGridder : public HighLevelGridderInterface {
   size_t _channelCount;
 };
 
+}  // namespace wsclean
+
 #endif
