@@ -10,6 +10,8 @@
 
 #include <fstream>
 
+namespace wsclean {
+
 class IdgConfiguration {
  public:
   static void Read(idg::api::Type& proxyType, int& bufferSize,
@@ -50,4 +52,6 @@ class IdgConfiguration {
   }
 };
 
+}  // namespace wsclean
+
 #endif
