@@ -5,6 +5,8 @@
 
 #include "cachedimageset.h"
 
+namespace wsclean {
+
 /**
  * @brief ImageAccessor implementation that internally uses CachedImageSet.
  */
@@ -102,4 +104,6 @@ class CachedImageAccessor : public aocommon::ImageAccessor {
   std::shared_ptr<const schaapcommon::facets::Facet> facet_;
 };
 
-#endif
\ No newline at end of file
+}  // namespace wsclean
+
+#endif
