@@ -7,6 +7,8 @@
 
 #include "../structures/primarybeamimageset.h"
 
+namespace wsclean {
+
 class Settings;
 
 /**
@@ -46,4 +48,6 @@ class ComponentListWriter {
   std::unique_ptr<radler::WorkTable> deconvolution_table_;
 };
 
+}  // namespace wsclean
+
 #endif
