@@ -2,22 +2,23 @@
 #define IMAGE_WEIGHT_CACHE_H
 
 #include "../structures/imageweights.h"
+#include "../structures/mslistitem.h"
 #include "../structures/weightmode.h"
 
-#include "../msproviders/msdatadescription.h"
-
 #include <aocommon/logger.h>
 
 #include <limits>
 #include <mutex>
 
+namespace wsclean {
+
 class ImageWeightCache {
  public:
   ImageWeightCache(const WeightMode& weightMode, size_t imageWidth,
                    size_t imageHeight, double pixelScaleX, double pixelScaleY,
                    double minUVInLambda, double maxUVInLambda,
                    double rankFilterLevel, size_t rankFilterSize,
-                   bool weightsAsTaper, size_t threadCount)
+                   bool weightsAsTaper)
       : _weightMode(weightMode),
         _imageWidth(imageWidth),
         _imageHeight(imageHeight),
@@ -33,7 +34,6 @@ class ImageWeightCache {
         _edgeTaperInLambda(0),
         _edgeTukeyTaperInLambda(0),
         _weightsAsTaper(weightsAsTaper),
-        _threadCount(threadCount),
         _currentWeightChannel(std::numeric_limits<size_t>::max()),
         _currentWeightInterval(std::numeric_limits<size_t>::max()) {}
 
@@ -47,11 +47,9 @@ class ImageWeightCache {
     _edgeTukeyTaperInLambda = edgeTukeyTaperInLambda;
   }
 
-  void SetThreadCount(size_t threadCount) { _threadCount = threadCount; }
-
-  std::shared_ptr<ImageWeights> Get(
-      const std::vector<std::unique_ptr<MSDataDescription>>& msList,
-      size_t outChannelIndex, size_t outIntervalIndex) {
+  std::shared_ptr<ImageWeights> Get(const std::vector<MsListItem>& msList,
+                                    size_t outChannelIndex,
+                                    size_t outIntervalIndex) {
     std::unique_lock<std::mutex> lock(_mutex);
     if (outChannelIndex != _currentWeightChannel ||
         outIntervalIndex != _currentWeightInterval) {
@@ -69,7 +67,6 @@ class ImageWeightCache {
     std::unique_ptr<ImageWeights> weights(new ImageWeights(
         _weightMode, _imageWidth, _imageHeight, _pixelScaleX, _pixelScaleY,
         _weightsAsTaper, _weightMode.SuperWeight()));
-    weights->SetThreadCount(_threadCount);
     return weights;
   };
 
@@ -85,13 +82,14 @@ class ImageWeightCache {
 
  private:
   std::unique_ptr<ImageWeights> recalculateWeights(
-      const std::vector<std::unique_ptr<MSDataDescription>>& msList) {
+      const std::vector<MsListItem>& msList) {
     aocommon::Logger::Info << "Precalculating weights for "
                            << _weightMode.ToString() << " weighting...\n";
     std::unique_ptr<ImageWeights> weights = MakeEmptyWeights();
-    for (size_t i = 0; i != msList.size(); ++i) {
-      std::unique_ptr<MSProvider> provider = msList[i]->GetProvider();
-      const MSSelection& selection = msList[i]->Selection();
+    for (const MsListItem& item : msList) {
+      std::unique_ptr<MSProvider> provider = item.ms_description->GetProvider();
+      const schaapcommon::reordering::MSSelection& selection =
+          item.ms_description->Selection();
       const aocommon::BandData selectedBand =
           selection.HasChannelRange()
               ? aocommon::BandData(provider->Band(),
@@ -143,9 +141,10 @@ class ImageWeightCache {
   double _edgeTukeyTaperInLambda;
   bool _weightsAsTaper;
   std::mutex _mutex;
-  size_t _threadCount;
 
   size_t _currentWeightChannel, _currentWeightInterval;
 };
 
+}  // namespace wsclean
+
 #endif
