@@ -8,6 +8,7 @@
 #include <aocommon/fits/fitswriter.h>
 #include <aocommon/logger.h>
 #include <aocommon/radeccoord.h>
+#include <aocommon/threadpool.h>
 #include <aocommon/units/angle.h>
 #include <aocommon/units/fluxdensity.h>
 
@@ -26,6 +27,8 @@ using aocommon::Logger;
 using aocommon::units::Angle;
 using aocommon::units::FluxDensity;
 
+namespace wsclean {
+
 namespace {
 void IncArgi(int& argi, int argc) {
   ++argi;
@@ -50,12 +53,18 @@ void CheckDeprecated(bool is_slave, const std::string& param,
   }
 }
 
-void PrintHeader() {
+void PrintHeader(bool print_commit_hash = false) {
   Logger::Info << "\n"
                   "WSClean version " WSCLEAN_VERSION_STR
-                  " (" WSCLEAN_VERSION_DATE
-                  ")\n"
-                  "This software package is released under the GPL version 3.\n"
+                  " (" WSCLEAN_VERSION_DATE ")\n";
+
+  if (print_commit_hash) {
+    std::string git_commit_hash = std::string(WSCLEAN_GIT_HASH).empty()
+                                      ? "not available"
+                                      : std::string(WSCLEAN_GIT_HASH);
+    Logger::Info << "Git commit hash: " << git_commit_hash << "\n";
+  }
+  Logger::Info << "This software package is released under the GPL version 3.\n"
                   "Author: André Offringa (offringa@gmail.com).\n\n";
 #ifndef NDEBUG
   Logger::Info
@@ -88,6 +97,12 @@ Options can be:
 -no-work-on-master
    In MPI runs, do not use the master for gridding. This may be useful if the
    resources such as memory of the master are limited.
+-channel-to-node <list>
+   In MPI runs, override the default channel-to-node assignment. The
+   comma-separated list must contain a node index for each output channel.
+-max-mpi-message-size <size>
+   In MPI runs, use a different maximum message size. The default value is 2 GB.
+   If size ends with g or m, the value is interpreted as gigabytes or megabytes.
 -mem <percentage>
    Limit memory usage to the given fraction of the total system memory. This is an approximate value.
    Default: 100.
@@ -106,6 +121,11 @@ Options can be:
    Default: only reorder when in channel imaging mode.
 -temp-dir <directory>
    Set the temporary directory used when reordering files. Default: same directory as input measurement set.
+-reuse-reordered
+   Reuse generated reordered temporary files (data, weights & metadata). These files can be generated by a
+   previous run of wsclean (with -save-reordered) or with DP3's 'reorder' step. Default: false.
+-save-reordered
+   Keep the reordered files instead of deleting them when finished. Implicitly sets -reorder.
 -update-model-required (default), and
 -no-update-model-required
    These two options specify whether the model data column is required to
@@ -247,10 +267,10 @@ Options can be:
    Split the bandwidth at the specified frequencies (in Hz) before the normal bandwidth
    division is performed. This can e.g. be useful for imaging multiple bands with irregular
    number of channels.
--no-small-inversion and -small-inversion
-   Perform inversion at the Nyquist resolution and upscale the image to the requested image size afterwards.
-   This speeds up inversion considerably, but makes aliasing slightly worse. This effect is
-   in most cases <1%. Default: on.
+-no-min-grid-resolution and -min-grid-resolution
+   Perform prediction and inversion at the Nyquist resolution and upscale the image to the requested image
+   size afterwards. This speeds up inversion and prediction considerably, but makes aliasing slightly worse.
+   This effect is in most cases <1%. Default: on.
 -make-psf
    Always make the psf, even when no cleaning is performed.
 -make-psf-only
@@ -268,7 +288,7 @@ Options can be:
    with antenna1 and antenna2 indices and the stddev per line, separated by spaces, e.g. "0 1 3.14".
 -idg-mode [cpu/gpu/hybrid]
    Sets the IDG mode. Default: cpu. Hybrid is recommended when a GPU is available.
-   
+
   ** GRIDDER-SPECIFIC SETTINGS **
 -wstack-nwlayers <nwlayers>
    Number of w-layers to use. Default: minimum suggested #w-layers for first MS.
@@ -284,10 +304,14 @@ Options can be:
 -wstack-kernel-size <size>
    Gridding antialiasing kernel size. Default: 7.
 -wstack-oversampling <factor>
-   Oversampling factor used during gridding. Default: 63.
+   Oversampling factor used during gridding. Default: 1023.
 -wgridder-accuracy <value>
    Set the w-gridding accuracy. Default: 1e-4
    Useful range: 1e-2 to 1e-6
+-compound-tasks
+   Schedule compound gridding tasks which contain all facets for a single image.
+-shared-facet-reads
+   When parallel gridding with multiple facets read data only once per compound gridding task into a shared data buffer and share this buffer for the gridders of all facets within the task. Implicitly sets -compound-tasks.
 
   ** A-TERM GRIDDING **
 -aterm-config <filename>
@@ -305,9 +329,16 @@ Options can be:
    Apply solutions from the provided (h5) file per facet when gridding facet based images.
    Provided file is assumed to be in H5Parm format.
    Filename is followed by a comma separated list of strings specifying which sol tabs from the provided H5Parm file are used.
--diagonal-solutions
-   Will apply diagonal (h5) solutions separately to the specified polarizations. This allows
-   separate correction of XX and YY while only making Stokes I images.
+-no-solution-directions-check
+   Disable the check that requires the number of solution directions to be equal to the number of directions in the region
+   file.
+-scalar-visibilities
+   Only read the visibilities as a single polarization (e.g. Stokes I). When imaging a single polarization and when the solutions
+   are also scalar, this option may make IO faster. By default, all four correlated visibilities (e.g. xx, xy, yx, yy) are read
+   when solutions (or beam gain) are applied, and the solutions are applied to those four.
+-diagonal-visibilities
+   Will read only diagonal visibilities (e.g. xx/yy) and apply solutions separately to these visibilities. This allows
+   separate correction of XX and YY while only making Stokes I images, without needing all the visibilities.
 -apply-facet-beam
    Apply beam gains to facet center when gridding facet based images or direction dependent psfs
 -facet-beam-update <seconds>
@@ -341,8 +372,13 @@ Options can be:
    all fields in the measurement set. Default: first field (id 0).
 -spws <list>
    Selects only the spws given in the list. list should be a comma-separated list of integers. Default: all spws.
--data-column <columnname>
+-data-column <column name>
    Default: CORRECTED_DATA if it exists, otherwise DATA will be used.
+-model-column <column name>
+   Column to which the predicted data is written. Default: MODEL_DATA.
+-model-storage-manager <type>
+   If a new model column needs to be written, create the model using this storage manager. Supported types:
+   "default" and "stokes-i".
 -maxuvw-m <meters>
 -minuvw-m <meters>
    Set the min/max baseline distance in meters.
@@ -357,7 +393,7 @@ Options can be:
 -niter <niter>
    Maximum number of clean iterations to perform. Default: 0 (=no cleaning)
 -nmiter <nmiter>
-   Maximum number of major clean (inversion/prediction) iterations. Default: 20.   A value of 0 means no limit.
+   Maximum number of major clean (inversion/prediction) iterations. Default: 12.   A value of 0 means no limit.
 -auto-threshold <sigma>
    Relative clean threshold. Estimate noise level using a robust estimator and stop at sigma x stddev.
 -abs-threshold <threshold>
@@ -371,6 +407,9 @@ Options can be:
 -local-rms
    Instead of using a single RMS for auto thresholding/masking, use a spatially varying
    RMS image.
+-local-rms-strength <strength>
+   A value between 0 and (normally) 1 that balances the effect of local RMS: a value of zero turns local RMS off,
+   whereas a value of 1 uses the local RMS directly to flatten the noise in the image. Default: 1.
 -local-rms-window
    Size of window for creating the RMS background map, in number of PSFs. Default: 25 psfs.
 -local-rms-method
@@ -399,7 +438,7 @@ but subtract components from individual channels.
 -no-fast-subminor
    Do not use the subminor loop optimization during (non-multiscale) cleaning. Default: use the optimization.
 -multiscale
-   Clean on different scales. This is a new algorithm. Default: off.
+   Clean on different scales. Default: off.
    This parameter invokes the optimized multiscale algorithm published by Offringa & Smirnov (2017).
 -multiscale-scale-bias
    Parameter to prevent cleaning small scales in the large-scale iterations. A lower
@@ -419,6 +458,8 @@ but subtract components from individual channels.
    A value of 0.1 might be more stable.
 -multiscale-convolution-padding <padding>
    Size of zero-padding for convolutions during the multi-scale cleaning. Default: 1.1
+-asp
+   Use the adaptive scale pixel algorithm.   
 -no-multiscale-fast-subminor
    Disable the 'fast subminor loop' optimization, that will only search a part of the
    image during the multi-scale subminor loop. The optimization is on by default.
@@ -520,7 +561,17 @@ std::vector<std::string> ParseStringList(const char* param) {
 size_t ParseSizeT(const char* param, const char* name) {
   char* endptr;
   errno = 0;
-  const long v = strtol(param, &endptr, 0);
+  long v = strtol(param, &endptr, 0);
+  if (*endptr == 'g' || *endptr == 'G') {
+    v *= 1024 * 1024 * 1024;
+    ++endptr;
+  } else if (*endptr == 'm' || *endptr == 'M') {
+    v *= 1024 * 1024;
+    ++endptr;
+  } else if (*endptr == 'k' || *endptr == 'K') {
+    v *= 1024;
+    ++endptr;
+  }
   if (*endptr != 0 || endptr == param || errno != 0) {
     std::ostringstream msg;
     msg << "Could not parse value '" << param << "' for parameter -" << name
@@ -582,13 +633,13 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
   int argi = 1;
   bool mfWeighting = false, noMFWeighting = false, dryRun = false;
   std::optional<double> atermKernelSize;
-  Logger::SetVerbosity(Logger::kNormalVerbosity);
+  Logger::SetVerbosity(aocommon::LogVerbosityLevel::kNormal);
   while (argi < argc && argv[argi][0] == '-') {
     const std::string param =
         argv[argi][1] == '-' ? (&argv[argi][2]) : (&argv[argi][1]);
     if (param == "version") {
       if (!isSlave) {
-        PrintHeader();
+        PrintHeader(true);
 #ifdef HAVE_EVERYBEAM
         Logger::Info << "EveryBeam is available.\n";
 #endif
@@ -605,9 +656,9 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       }
       return false;
     } else if (param == "quiet") {
-      Logger::SetVerbosity(Logger::kQuietVerbosity);
+      Logger::SetVerbosity(aocommon::LogVerbosityLevel::kQuiet);
     } else if (param == "v" || param == "verbose") {
-      Logger::SetVerbosity(Logger::kVerboseVerbosity);
+      Logger::SetVerbosity(aocommon::LogVerbosityLevel::kVerbose);
     } else if (param == "log-time") {
       Logger::SetLogTime(true);
     } else if (param == "temp-dir") {
@@ -635,11 +686,10 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "subtract-model") {
       settings.subtractModel = true;
     } else if (param == "size") {
-      size_t width = ParseSizeT(argv[argi + 1], "size"),
-             height = ParseSizeT(argv[argi + 2], "size");
-      settings.trimmedImageWidth = width;
-      settings.trimmedImageHeight = height;
-      argi += 2;
+      IncArgi(argi, argc);
+      settings.trimmedImageWidth = ParseSizeT(argv[argi], "size");
+      IncArgi(argi, argc);
+      settings.trimmedImageHeight = ParseSizeT(argv[argi], "size");
     } else if (param == "padding") {
       IncArgi(argi, argc);
       settings.imagePadding = ParseDouble(argv[argi], 1.0, "padding");
@@ -663,9 +713,10 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "nwlayers-for-size" ||
                param == "wstack-nwlayers-for-size") {
       CheckDeprecated(isSlave, param, "wstack-nwlayers-for-size");
-      settings.widthForNWCalculation = ParseSizeT(argv[argi + 1], param);
-      settings.heightForNWCalculation = ParseSizeT(argv[argi + 2], param);
-      argi += 2;
+      IncArgi(argi, argc);
+      settings.widthForNWCalculation = ParseSizeT(argv[argi], param);
+      IncArgi(argi, argc);
+      settings.heightForNWCalculation = ParseSizeT(argv[argi], param);
     } else if (param == "grid-mode" || param == "wstack-grid-mode") {
       CheckDeprecated(isSlave, param, "wstack-grid-mode");
       IncArgi(argi, argc);
@@ -731,6 +782,10 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
           argv[argi], "absolute threshold parameter", FluxDensity::kJansky);
     } else if (param == "local-rms") {
       settings.localRMSMethod = radler::LocalRmsMethod::kRmsWindow;
+    } else if (param == "local-rms-strength") {
+      IncArgi(argi, argc);
+      settings.localRMSStrength =
+          ParseDouble(argv[argi], 0.0, "local-rms-strength");
     } else if (param == "local-rms-window") {
       IncArgi(argi, argc);
       settings.localRMSMethod = radler::LocalRmsMethod::kRmsWindow;
@@ -742,7 +797,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       settings.localRMSImage = argv[argi];
     } else if (param == "local-rms-method") {
       IncArgi(argi, argc);
-      std::string method = argv[argi];
+      const std::string method = argv[argi];
       if (method == "rms")
         settings.localRMSMethod = radler::LocalRmsMethod::kRmsWindow;
       else if (method == "rms-with-min")
@@ -752,6 +807,13 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "data-column") {
       IncArgi(argi, argc);
       settings.dataColumnName = argv[argi];
+    } else if (param == "model-column") {
+      IncArgi(argi, argc);
+      settings.modelColumnName = argv[argi];
+    } else if (param == "model-storage-manager") {
+      IncArgi(argi, argc);
+      settings.modelStorageManager =
+          schaapcommon::reordering::GetStorageManagerType(argv[argi]);
     } else if (param == "pol") {
       IncArgi(argi, argc);
       settings.polarizations = aocommon::Polarization::ParseList(argv[argi]);
@@ -818,9 +880,10 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       IncArgi(argi, argc);
       settings.primaryBeamGridSize = ParseSizeT(argv[argi], "pb-grid-size");
     } else if (param == "dd-psf-grid") {
-      settings.ddPsfGridWidth = ParseSizeT(argv[argi + 1], "dd-psf-grid");
-      settings.ddPsfGridHeight = ParseSizeT(argv[argi + 2], "dd-psf-grid");
-      argi += 2;
+      IncArgi(argi, argc);
+      settings.ddPsfGridWidth = ParseSizeT(argv[argi], "dd-psf-grid");
+      IncArgi(argi, argc);
+      settings.ddPsfGridHeight = ParseSizeT(argv[argi], "dd-psf-grid");
     } else if (param == "negative") {
       settings.allowNegativeComponents = true;
     } else if (param == "no-negative") {
@@ -860,30 +923,36 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "name") {
       IncArgi(argi, argc);
       settings.prefixName = argv[argi];
-    } else if (param == "small-inversion") {
-      settings.smallInversion = true;
-    } else if (param == "no-small-inversion") {
-      settings.smallInversion = false;
+    } else if (param == "min-grid-resolution" || param == "small-inversion") {
+      CheckDeprecated(isSlave, param, "min-grid-resolution");
+      settings.minGridResolution = true;
+    } else if (param == "no-min-grid-resolution" ||
+               param == "no-small-inversion") {
+      CheckDeprecated(isSlave, param, "no-min-grid-resolution");
+      settings.minGridResolution = false;
     } else if (param == "interval") {
-      settings.startTimestep = ParseSizeT(argv[argi + 1], "interval");
-      settings.endTimestep = ParseSizeT(argv[argi + 2], "interval");
-      argi += 2;
+      IncArgi(argi, argc);
+      settings.startTimestep = ParseSizeT(argv[argi], "interval");
+      IncArgi(argi, argc);
+      settings.endTimestep = ParseSizeT(argv[argi], "interval");
     } else if (param == "intervals-out") {
       IncArgi(argi, argc);
       settings.intervalsOut = atoi(argv[argi]);
     } else if (param == "even-timesteps") {
-      settings.evenOddTimesteps = MSSelection::EvenTimesteps;
+      settings.evenOddTimesteps = MSSelection::kEvenTimesteps;
     } else if (param == "odd-timesteps") {
-      settings.evenOddTimesteps = MSSelection::OddTimesteps;
+      settings.evenOddTimesteps = MSSelection::kOddTimesteps;
     } else if (param == "channel-range") {
-      settings.startChannel = ParseSizeT(argv[argi + 1], "channel-range");
-      settings.endChannel = ParseSizeT(argv[argi + 2], "channel-range");
-      argi += 2;
+      IncArgi(argi, argc);
+      settings.startChannel = ParseSizeT(argv[argi], "channel-range");
+      IncArgi(argi, argc);
+      settings.endChannel = ParseSizeT(argv[argi], "channel-range");
     } else if (param == "shift") {
       settings.hasShift = true;
-      settings.shiftRA = aocommon::RaDecCoord::ParseRA(argv[argi + 1]);
-      settings.shiftDec = aocommon::RaDecCoord::ParseDec(argv[argi + 2]);
-      argi += 2;
+      IncArgi(argi, argc);
+      settings.shiftRA = aocommon::RaDecCoord::ParseRA(argv[argi]);
+      IncArgi(argi, argc);
+      settings.shiftDec = aocommon::RaDecCoord::ParseDec(argv[argi]);
     } else if (param == "channels-out") {
       IncArgi(argi, argc);
       settings.channelsOut = ParseSizeT(argv[argi], "channels-out");
@@ -917,12 +986,12 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       // mfs was renamed to mf in wsclean 2.7
       CheckDeprecated(isSlave, param, "no-mf-weighting");
     } else if (param == "spectral-correction") {
+      IncArgi(argi, argc);
       settings.spectralCorrectionFrequency =
-          ParseDouble(argv[argi + 1], 0.0, "spectral-correction", false);
-      aocommon::UVector<double> list =
-          NumberList::ParseDoubleList(argv[argi + 2]);
+          ParseDouble(argv[argi], 0.0, "spectral-correction", false);
+      IncArgi(argi, argc);
+      aocommon::UVector<double> list = NumberList::ParseDoubleList(argv[argi]);
       settings.spectralCorrection.assign(list.begin(), list.end());
-      argi += 2;
     } else if (param == "taper-gaussian") {
       IncArgi(argi, argc);
       double taperBeamSize =
@@ -982,6 +1051,8 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
           ParseDouble(argv[argi], 1.0, "multiscale-convolution-padding");
     } else if (param == "no-multiscale-fast-subminor") {
       settings.multiscaleFastSubMinorLoop = false;
+    } else if (param == "asp") {
+      settings.algorithmType = radler::AlgorithmType::kAdaptiveScalePixel;
     } else if (param == "weighting-rank-filter") {
       IncArgi(argi, argc);
       settings.rankFilterLevel =
@@ -1041,7 +1112,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
     } else if (param == "field") {
       IncArgi(argi, argc);
       if (argv[argi] == std::string("all"))
-        settings.fieldIds.assign(1, MSSelection::ALL_FIELDS);
+        settings.fieldIds.assign(1, MSSelection::kAllFields);
       else {
         aocommon::UVector<int> list = NumberList::ParseIntList(argv[argi]);
         settings.fieldIds.assign(list.begin(), list.end());
@@ -1054,9 +1125,9 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       IncArgi(argi, argc);
       std::string weightArg = argv[argi];
       if (weightArg == "natural")
-        settings.weightMode = WeightMode(WeightMode::NaturalWeighted);
+        settings.weightMode = WeightMode(WeightClass::Natural);
       else if (weightArg == "uniform")
-        settings.weightMode = WeightMode(WeightMode::UniformWeighted);
+        settings.weightMode = WeightMode(WeightClass::Uniform);
       else if (weightArg == "briggs") {
         IncArgi(argi, argc);
         settings.weightMode =
@@ -1072,10 +1143,12 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
         settings.mode = Settings::RestoreMode;
       else
         settings.mode = Settings::RestoreListMode;
-      settings.restoreInput = argv[argi + 1];
-      settings.restoreModel = argv[argi + 2];
-      settings.restoreOutput = argv[argi + 3];
-      argi += 3;
+      IncArgi(argi, argc);
+      settings.restoreInput = argv[argi];
+      IncArgi(argi, argc);
+      settings.restoreModel = argv[argi];
+      IncArgi(argi, argc);
+      settings.restoreOutput = argv[argi];
     } else if (param == "beam-size") {
       IncArgi(argi, argc);
       double beam = Angle::Parse(argv[argi], "beam size", Angle::kArcseconds);
@@ -1083,16 +1156,15 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       settings.manualBeamMinorSize = beam;
       settings.manualBeamPA = 0.0;
     } else if (param == "beam-shape") {
-      double beamMaj = Angle::Parse(argv[argi + 1], "beam shape, major axis",
-                                    Angle::kArcseconds);
-      double beamMin = Angle::Parse(argv[argi + 2], "beam shape, minor axis",
-                                    Angle::kArcseconds);
-      double beamPA = Angle::Parse(argv[argi + 3], "beam shape, position angle",
-                                   Angle::kDegrees);
-      argi += 3;
-      settings.manualBeamMajorSize = beamMaj;
-      settings.manualBeamMinorSize = beamMin;
-      settings.manualBeamPA = beamPA;
+      IncArgi(argi, argc);
+      settings.manualBeamMajorSize = Angle::Parse(
+          argv[argi], "beam shape, major axis", Angle::kArcseconds);
+      IncArgi(argi, argc);
+      settings.manualBeamMinorSize = Angle::Parse(
+          argv[argi], "beam shape, minor axis", Angle::kArcseconds);
+      IncArgi(argi, argc);
+      settings.manualBeamPA = Angle::Parse(
+          argv[argi], "beam shape, position angle", Angle::kDegrees);
     } else if (param == "fit-beam") {
       settings.fittedBeam = true;
     } else if (param == "no-fit-beam") {
@@ -1108,6 +1180,14 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       settings.circularBeam = true;
     } else if (param == "elliptical-beam") {
       settings.circularBeam = false;
+    } else if (param == "reuse-reordered") {
+      settings.reuseReorder = true;
+      settings.forceReorder = true;
+      settings.forceNoReorder = false;
+    } else if (param == "save-reordered") {
+      settings.saveReorder = true;
+      settings.forceReorder = true;
+      settings.forceNoReorder = false;
     } else if (param == "reorder") {
       settings.forceReorder = true;
       settings.forceNoReorder = false;
@@ -1130,6 +1210,14 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       settings.parallelGridding = ParseSizeT(argv[argi], "parallel-gridding");
     } else if (param == "no-work-on-master") {
       settings.masterDoesWork = false;
+    } else if (param == "channel-to-node") {
+      IncArgi(argi, argc);
+      const aocommon::UVector<int> list = NumberList::ParseIntList(argv[argi]);
+      settings.channelToNode.assign(list.begin(), list.end());
+    } else if (param == "max-mpi-message-size") {
+      IncArgi(argi, argc);
+      settings.maxMpiMessageSize =
+          ParseSizeT(argv[argi], "max-mpi-message-size");
     } else if (param == "mem") {
       IncArgi(argi, argc);
       settings.memFraction = ParseDouble(argv[argi], 0.0, "mem", false) / 100.0;
@@ -1166,6 +1254,11 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       IncArgi(argi, argc);
       settings.simulateNoise = true;
       settings.simulatedBaselineNoiseFilename = argv[argi];
+    } else if (param == "compound-tasks") {
+      settings.compound_tasks = true;
+    } else if (param == "shared-facet-reads") {
+      settings.compound_tasks = true;
+      settings.shared_facet_reads = true;
     } else if (param == "aterm-config") {
       IncArgi(argi, argc);
       settings.atermConfigFilename = argv[argi];
@@ -1189,8 +1282,15 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
             "List of solution tables (soltabs) should contain at most two "
             "entries.");
       }
-    } else if (param == "diagonal-solutions") {
-      settings.diagonalSolutions = true;
+    } else if (param == "no-solution-directions-check") {
+      settings.solutionDirectionsCheck = false;
+    } else if (param == "scalar-visibilities") {
+      settings.visibilityReadMode = VisibilityReadMode::kScalar;
+    } else if (param == "diagonal-solutions" ||
+               param == "diagonal-visibilities") {
+      // Deprecated July 2024
+      CheckDeprecated(isSlave, param, "diagonal-visibilities");
+      settings.visibilityReadMode = VisibilityReadMode::kDiagonal;
     } else if (param == "apply-facet-beam") {
       settings.applyFacetBeam = true;
     } else if (param == "facet-beam-update") {
@@ -1218,7 +1318,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
       Deprecated(isSlave, param, "gridder");
       settings.gridderType = GridderType::DirectFT;
       settings.imagePadding = 1.0;
-      settings.smallInversion = false;
+      settings.minGridResolution = false;
     } else if (param == "direct-ft-precision") {
       IncArgi(argi, argc);
       std::string precStr = argv[argi];
@@ -1242,7 +1342,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
             "recompile WSClean");
 #endif
         settings.gridderType = GridderType::IDG;
-        settings.smallInversion = false;
+        settings.minGridResolution = false;
       } else if (gridder_str == "wgridder") {
         settings.gridderType = GridderType::WGridder;
       } else if (gridder_str == "tuned-wgridder") {
@@ -1262,7 +1362,7 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
           "recompile WSClean");
 #endif
       settings.gridderType = GridderType::IDG;
-      settings.smallInversion = false;
+      settings.minGridResolution = false;
     } else if (param == "idg-mode") {
       IncArgi(argi, argc);
       std::string mode =
@@ -1303,6 +1403,13 @@ bool CommandLine::ParseWithoutValidation(WSClean& wsclean, int argc,
   // and possibly set to quiet.
   if (!isSlave) PrintHeader();
 
+  if (settings.shared_facet_reads &&
+      settings.gridderType != GridderType::WGridder) {
+    throw std::runtime_error(
+        "-shared-facet-reads are currently only compatible with -gridder "
+        "wgridder");
+  }
+
   const size_t defaultAtermSize = settings.atermConfigFilename.empty() ? 5 : 16;
   settings.atermKernelSize = atermKernelSize.value_or(defaultAtermSize);
 
@@ -1335,6 +1442,10 @@ void CommandLine::Validate(WSClean& wsclean) {
 
 void CommandLine::Run(class WSClean& wsclean) {
   const Settings& settings = wsclean.GetSettings();
+  // Implicitly initialise cfitsio to prevent a race condition that can
+  // otherwise occur inside `fits_open_file` and other functions.
+  fits_init_cfitsio();
+  aocommon::ThreadPool::GetInstance().SetNThreads(settings.threadCount);
   switch (settings.mode) {
     case Settings::RestoreMode:
       WSCFitsWriter::Restore(settings);
@@ -1350,3 +1461,5 @@ void CommandLine::Run(class WSClean& wsclean) {
       break;
   }
 }
+
+}  // namespace wsclean
