@@ -6,6 +6,8 @@
 
 #include "settings.h"
 
+namespace wsclean {
+
 class CommandLine {
  public:
   /**
@@ -37,4 +39,6 @@ class CommandLine {
   static void Run(class WSClean& wsclean);
 };
 
+}  // namespace wsclean
+
 #endif
