@@ -3,6 +3,8 @@
 
 #include <string>
 
+namespace wsclean {
+
 class ProgressBar {
  public:
   explicit ProgressBar(const std::string& taskDescription);
@@ -17,4 +19,6 @@ class ProgressBar {
   int _displayedDots;
 };
 
+}  // namespace wsclean
+
 #endif
