@@ -1,6 +1,8 @@
 #ifndef CALCULATE_FFT_SIZE_H
 #define CALCULATE_FFT_SIZE_H
 
+namespace wsclean {
+
 namespace detail {
 inline bool hasLowPrimeFactors(size_t number) {
   while (number > 7) {
@@ -31,4 +33,6 @@ inline void CalculateFFTSize(size_t size, double pixelScale, double beamSize,
   if (optimalSize > size) optimalSize = size;
 }
 
+}  // namespace wsclean
+
 #endif
