@@ -9,7 +9,7 @@
 
 #include "../model/model.h"
 
-namespace renderer {
+namespace wsclean::renderer {
 
 /**
  * @brief Struct collecting the relevant image coordinate settings for
@@ -54,13 +54,15 @@ struct ImageCoordinateSettings {
  * @param polarization Polarization enum.
  * @param thread_count Numbers of threads to use.
  */
-void RestoreWithEllipticalBeam(
-    aocommon::Image& image, const ImageCoordinateSettings& image_settings,
-    const Model& model, long double beam_major_axis,
-    long double beam_minor_axis, long double beam_position_angle,
-    long double start_frequency, long double end_frequency,
-    aocommon::PolarizationEnum polarization, size_t thread_count);
+void RestoreWithEllipticalBeam(aocommon::Image& image,
+                               const ImageCoordinateSettings& image_settings,
+                               const Model& model, long double beam_major_axis,
+                               long double beam_minor_axis,
+                               long double beam_position_angle,
+                               long double start_frequency,
+                               long double end_frequency,
+                               aocommon::PolarizationEnum polarization);
 
-}  // namespace renderer
+}  // namespace wsclean::renderer
 
 #endif
