@@ -16,6 +16,8 @@
 #include <stdexcept>
 #include <fstream>
 
+namespace wsclean {
+
 class BBSModel {
  public:
   /**
@@ -289,4 +291,6 @@ class BBSModel {
   };
 };
 
+}  // namespace wsclean
+
 #endif
