@@ -8,6 +8,8 @@
 #include "measurement.h"
 #include "spectralenergydistribution.h"
 
+namespace wsclean {
+
 class MeasuredSED : public SpectralEnergyDistribution {
  private:
   typedef std::map<long double, Measurement> FluxMap;
@@ -527,4 +529,6 @@ class MeasuredSED : public SpectralEnergyDistribution {
   FluxMap _measurements;
 };
 
+}  // namespace wsclean
+
 #endif
