@@ -11,6 +11,8 @@
 #include <fstream>
 #include <stdexcept>
 
+namespace wsclean {
+
 class ModelParser : private Tokenizer {
  public:
   ModelParser() : _fileVersion1_0(false) {}
@@ -272,4 +274,6 @@ class ModelParser : private Tokenizer {
   }
 };
 
+}  // namespace wsclean
+
 #endif
