@@ -13,7 +13,9 @@
 #include <limits>
 #include <stdexcept>
 
-//#define EXTRA_ASSERTIONS 1
+// #define EXTRA_ASSERTIONS 1
+
+namespace wsclean {
 
 class SpectralEnergyDistribution {
  public:
@@ -111,4 +113,6 @@ class SpectralEnergyDistribution {
   }
 };
 
+}  // namespace wsclean
+
 #endif
