@@ -3,20 +3,24 @@
 
 #include "msrowprovider.h"
 
+namespace wsclean {
+
 class DirectMSRowProvider : public MSRowProvider {
  public:
   DirectMSRowProvider(const string& msPath, const MSSelection& selection,
                       const std::map<size_t, size_t>& selectedDataDescIds,
-                      const std::string& dataColumnName, bool requireModel)
+                      const std::string& dataColumnName,
+                      const std::string& modelColumnName, bool requireModel)
       : MSRowProvider(msPath, selection, selectedDataDescIds, dataColumnName,
-                      requireModel) {}
+                      modelColumnName, requireModel) {}
 
   explicit DirectMSRowProvider(
       const casacore::MeasurementSet& ms, const MSSelection& selection,
       const std::map<size_t, size_t>& selected_data_description_ids,
-      const std::string& data_column_name, bool require_model)
+      const std::string& data_column_name, const std::string& model_column_name,
+      bool require_model)
       : MSRowProvider(ms, selection, selected_data_description_ids,
-                      data_column_name, require_model) {}
+                      data_column_name, model_column_name, require_model) {}
 
   virtual void ReadData(DataArray& data, FlagArray& flags, WeightArray& weights,
                         double& u, double& v, double& w, uint32_t& dataDescId,
@@ -24,8 +28,8 @@ class DirectMSRowProvider : public MSRowProvider {
                         uint32_t& fieldId, double& time) override;
 
   virtual void ReadModel(DataArray& model) final override;
-
- private:
 };
 
+}  // namespace wsclean
+
 #endif
