@@ -12,6 +12,8 @@ namespace casacore {
 class MeasurementSet;
 }  // namespace casacore
 
+namespace wsclean {
+
 /** Helper class to provide the columns in a measurement set. */
 struct MsColumns {
   explicit MsColumns(const casacore::MeasurementSet& ms,
@@ -28,4 +30,6 @@ struct MsColumns {
   casacore::ScalarColumn<int> data_description_id;
 };
 
+}  // namespace wsclean
+
 #endif  //  MSPROVIDERS_MSCOLUMNS_H
