@@ -1,6 +1,10 @@
 #include "contiguousmsreader.h"
 #include "../contiguousms.h"
 
+#include <schaapcommon/reordering/reordering.h>
+
+namespace wsclean {
+
 ContiguousMSReader::ContiguousMSReader(ContiguousMS* contiguousms)
     : MSReader(contiguousms),
       _currentInputRow(contiguousms->_startRow - 1),
@@ -29,7 +33,7 @@ bool ContiguousMSReader::CurrentRowAvailable() {
   casacore::Vector<double> uvw = contiguousms._uvwColumn(_currentInputRow);
 
   while (!contiguousms._selection.IsSelected(fieldId, _currentInputTimestep, a1,
-                                             a2, uvw) ||
+                                             a2, uvw.data()) ||
          dataDescId != contiguousms._dataDescId) {
     ++_currentInputRow;
     if (_currentInputRow >= contiguousms._endRow) return false;
@@ -78,7 +82,7 @@ void ContiguousMSReader::NextInputRow() {
       _currentInputTime = contiguousms._timeColumn(_currentInputRow);
     }
   } while (!contiguousms._selection.IsSelected(fieldId, _currentInputTimestep,
-                                               a1, a2, uvw) ||
+                                               a1, a2, uvw.data()) ||
            (dataDescId != contiguousms._dataDescId));
 }
 
@@ -120,9 +124,9 @@ void ContiguousMSReader::ReadData(std::complex<float>* buffer) {
     endChannel =
         contiguousms._bandData[contiguousms._dataDescId].ChannelCount();
   }
-  MSProvider::CopyData(
+  schaapcommon::reordering::ExtractData(
       buffer, startChannel, endChannel, contiguousms._inputPolarizations,
-      contiguousms._dataArray, contiguousms._outputPolarization);
+      contiguousms._dataArray.data(), contiguousms._outputPolarization);
 }
 
 void ContiguousMSReader::ReadModel(std::complex<float>* buffer) {
@@ -141,9 +145,9 @@ void ContiguousMSReader::ReadModel(std::complex<float>* buffer) {
     endChannel =
         contiguousms._bandData[contiguousms._dataDescId].ChannelCount();
   }
-  MSProvider::CopyData(
+  schaapcommon::reordering::ExtractData(
       buffer, startChannel, endChannel, contiguousms._inputPolarizations,
-      contiguousms._modelArray, contiguousms._outputPolarization);
+      contiguousms._modelArray.data(), contiguousms._outputPolarization);
 }
 
 void ContiguousMSReader::ReadWeights(float* buffer) {
@@ -161,10 +165,10 @@ void ContiguousMSReader::ReadWeights(float* buffer) {
     endChannel =
         contiguousms._bandData[contiguousms._dataDescId].ChannelCount();
   }
-  MSProvider::CopyWeights(
+  schaapcommon::reordering::ExtractWeights(
       buffer, startChannel, endChannel, contiguousms._inputPolarizations,
-      contiguousms._dataArray, contiguousms._weightSpectrumArray,
-      contiguousms._flagArray, contiguousms._outputPolarization);
+      contiguousms._dataArray.data(), contiguousms._weightSpectrumArray.data(),
+      contiguousms._flagArray.data(), contiguousms._outputPolarization);
 }
 
 void ContiguousMSReader::WriteImagingWeights(const float* buffer) {
@@ -186,10 +190,10 @@ void ContiguousMSReader::WriteImagingWeights(const float* buffer) {
 
   _imagingWeightsColumn->get(_currentInputRow,
                              contiguousms._imagingWeightSpectrumArray);
-  MSProvider::ReverseCopyWeights(contiguousms._imagingWeightSpectrumArray,
-                                 startChannel, endChannel,
-                                 contiguousms._inputPolarizations, buffer,
-                                 contiguousms._outputPolarization);
+  schaapcommon::reordering::StoreWeights(
+      contiguousms._imagingWeightSpectrumArray.data(), startChannel, endChannel,
+      contiguousms._inputPolarizations, buffer,
+      contiguousms._outputPolarization);
   _imagingWeightsColumn->put(_currentInputRow,
                              contiguousms._imagingWeightSpectrumArray);
 }
@@ -227,3 +231,5 @@ void ContiguousMSReader::readModel() {
     _isModelRead = true;
   }
 }
+
+}  // namespace wsclean
