@@ -3,6 +3,8 @@
 
 #include "msreader.h"
 
+namespace wsclean {
+
 class ContiguousMS;
 
 class ContiguousMSReader final : public MSReader {
@@ -45,4 +47,6 @@ class ContiguousMSReader final : public MSReader {
   void readModel();
 };
 
+}  // namespace wsclean
+
 #endif
