@@ -4,6 +4,8 @@
 #include "msreader.h"
 #include "../timestepbuffer.h"
 
+namespace wsclean {
+
 class TimestepBufferReader final : public MSReader {
  public:
   TimestepBufferReader(TimestepBuffer* timestepBuffer);
@@ -57,4 +59,6 @@ class TimestepBufferReader final : public MSReader {
   std::vector<TimestepBuffer::RowData> _buffer;
 };
 
+}  // namespace wsclean
+
 #endif
