@@ -5,12 +5,17 @@
 
 #include <casacore/ms/MeasurementSets/MeasurementSet.h>
 
+using schaapcommon::reordering::MSSelection;
+
+namespace wsclean {
+
 MSRowProvider::MSRowProvider(
     const string& msPath, const MSSelection& selection,
     const std::map<size_t, size_t>& selectedDataDescIds,
-    const std::string& dataColumnName, bool requireModel)
+    const std::string& dataColumnName, const std::string& model_column_name,
+    bool requireModel)
     : MsRowProviderBase(casacore::MeasurementSet(msPath), selection,
-                        dataColumnName),
+                        dataColumnName, model_column_name),
       _selectedDataDescIds(selectedDataDescIds),
       _requireModel(requireModel) {
   Initialize();
@@ -19,8 +24,9 @@ MSRowProvider::MSRowProvider(
 MSRowProvider::MSRowProvider(
     const casacore::MeasurementSet& ms, const MSSelection& selection,
     const std::map<size_t, size_t>& selected_data_description_ids,
-    const std::string& data_column_name, bool require_model)
-    : MsRowProviderBase(ms, selection, data_column_name),
+    const std::string& data_column_name, const std::string& model_column_name,
+    bool require_model)
+    : MsRowProviderBase(ms, selection, data_column_name, model_column_name),
       _selectedDataDescIds(selected_data_description_ids),
       _requireModel(require_model) {
   Initialize();
@@ -33,8 +39,8 @@ void MSRowProvider::Initialize() {
         "processing.");
 
   if (_requireModel)
-    _modelColumn.reset(new casacore::ArrayColumn<casacore::Complex>(
-        Ms(), casacore::MS::columnName(casacore::MSMainEnums::MODEL_DATA)));
+    _modelColumn.reset(
+        new casacore::ArrayColumn<casacore::Complex>(Ms(), ModelColumnName()));
 
   MsColumns& columns = Columns();
   _msHasWeights =
@@ -98,7 +104,7 @@ bool MSRowProvider::isCurrentRowSelected(int fieldId, int a1, int a2) const {
       _selectedDataDescIds.find(_currentDataDescId);
   bool isDataDescIdSelected = dataDescIdIter != _selectedDataDescIds.end();
   return Selection().IsSelected(fieldId, _currentTimestep, a1, a2,
-                                _currentUVWArray) &&
+                                _currentUVWArray.data()) &&
          isDataDescIdSelected;
 }
 
@@ -112,3 +118,5 @@ void MSRowProvider::getCurrentWeights(WeightArray& weights,
     MSProvider::ExpandScalarWeights(_scratchWeightScalarArray, weights);
   }
 }
+
+}  // namespace wsclean
