@@ -12,6 +12,8 @@
 #include <memory>
 #include <vector>
 
+namespace wsclean {
+
 /**
  * An MSRowProvider provides the selected rows of a data set.
  */
@@ -19,12 +21,14 @@ class MSRowProvider : public MsRowProviderBase {
  public:
   MSRowProvider(const string& msPath, const MSSelection& selection,
                 const std::map<size_t, size_t>& selectedDataDescIds,
-                const std::string& dataColumnName, bool requireModel);
+                const std::string& dataColumnName,
+                const std::string& model_column_name, bool requireModel);
 
   explicit MSRowProvider(
       const casacore::MeasurementSet& ms, const MSSelection& selection,
       const std::map<size_t, size_t>& selected_data_description_ids,
-      const std::string& data_column_name, bool require_model);
+      const std::string& data_column_name, const std::string& model_column_name,
+      bool require_model);
 
   bool AtEnd() const override { return _currentRow == EndRow(); }
 
@@ -68,4 +72,6 @@ class MSRowProvider : public MsRowProviderBase {
   void Initialize();
 };
 
+}  // namespace wsclean
+
 #endif
