@@ -14,6 +14,7 @@ namespace casacore {
 class MeasurementSet;
 }  // namespace casacore
 
+namespace wsclean {
 /**
  * Helper to get the Measurement set's weight.
  *
@@ -41,4 +42,6 @@ class MsWeightColumn {
   Type type_{Type::kSpectrum};
 };
 
+}  // namespace wsclean
+
 #endif  // MSPROVIDERS_MSWEIGHTCOLUMN_H
