@@ -7,11 +7,14 @@
 #include <random>
 #include <string>
 
+namespace wsclean {
+
 class NoiseMSRowProvider : public DirectMSRowProvider {
  public:
   NoiseMSRowProvider(const string& msPath, const MSSelection& selection,
                      const std::map<size_t, size_t>& selectedDataDescIds,
-                     const std::string& dataColumnName, bool requireModel);
+                     const std::string& dataColumnName,
+                     const std::string& modelColumnName, bool requireModel);
 
   void SetNoiseLevel(double noiseStdDevJy);
 
@@ -40,4 +43,6 @@ class NoiseMSRowProvider : public DirectMSRowProvider {
   NoiseMap _noiseMap;
 };
 
+}  // namespace wsclean
+
 #endif
