@@ -1,7 +1,11 @@
 #include "timestepbuffer.h"
 #include "msreaders/timestepbufferreader.h"
 
+namespace wsclean {
+
 std::unique_ptr<MSReader> TimestepBuffer::MakeReader() {
   std::unique_ptr<MSReader> reader(new TimestepBufferReader(this));
   return reader;
-}
\ No newline at end of file
+}
+
+}  // namespace wsclean
