@@ -5,16 +5,7 @@
 
 #include "../idg/averagebeam.h"
 
-GriddingResult::GriddingResult()
-    : startTime(0.0),
-      beamSize(0.0),
-      imageWeight(0.0),
-      normalizationFactor(0.0),
-      actualWGridSize(0),
-      griddedVisibilityCount(0),
-      effectiveGriddedVisibilityCount(0),
-      visibilityWeightSum(0),
-      cache() {}
+namespace wsclean {
 
 GriddingResult::GriddingResult(GriddingResult&& source) noexcept = default;
 GriddingResult::~GriddingResult() = default;
@@ -22,29 +13,46 @@ GriddingResult& GriddingResult::operator=(GriddingResult&& rhs) noexcept =
     default;
 
 void GriddingResult::Serialize(aocommon::SerialOStream& stream) const {
-  stream.ObjectVector(images)
+  stream.UInt32(unique_id)
       .Double(startTime)
       .Double(beamSize)
+      .UInt64(griddedVisibilityCount)
+      .Double(visibilityWeightSum)
+      .ObjectVector(facets);
+}
+
+void GriddingResult::Unserialize(aocommon::SerialIStream& stream) {
+  stream.UInt32(unique_id)
+      .Double(startTime)
+      .Double(beamSize)
+      .UInt64(griddedVisibilityCount)
+      .Double(visibilityWeightSum)
+      .ObjectVector(facets);
+}
+
+void GriddingResult::FacetData::Serialize(
+    aocommon::SerialOStream& stream) const {
+  stream.ObjectVector(images)
       .Double(imageWeight)
       .Double(normalizationFactor)
       .UInt64(actualWGridSize)
-      .UInt64(griddedVisibilityCount)
       .Double(effectiveGriddedVisibilityCount)
-      .Double(visibilityWeightSum)
+      .Object(averageCorrection)
+      .Object(averageBeamCorrection)
       .Ptr(cache)
       .Ptr(averageBeam);
 }
 
-void GriddingResult::Unserialize(aocommon::SerialIStream& stream) {
+void GriddingResult::FacetData::Unserialize(aocommon::SerialIStream& stream) {
   stream.ObjectVector(images)
-      .Double(startTime)
-      .Double(beamSize)
       .Double(imageWeight)
       .Double(normalizationFactor)
       .UInt64(actualWGridSize)
-      .UInt64(griddedVisibilityCount)
       .Double(effectiveGriddedVisibilityCount)
-      .Double(visibilityWeightSum)
+      .Object(averageCorrection)
+      .Object(averageBeamCorrection)
       .Ptr(cache)
       .Ptr(averageBeam);
 }
+
+}  // namespace wsclean
