@@ -20,7 +20,7 @@ MWA_COEFF_H5=mwa_full_embedded_element_pattern.h5
 
 if [ ! -f ${MWA_COEFF_H5} ] ; then
 
-    wget -q www.astron.nl/citt/EveryBeam/mwa_full_embedded_element_pattern.tar.bz2 -O ${MWA_COEFF_ARCHIVE}
+    wget -q https://support.astron.nl/software/ci_data/EveryBeam/mwa_full_embedded_element_pattern.tar.bz2 -O ${MWA_COEFF_ARCHIVE}
 
     tar -xjf $MWA_COEFF_ARCHIVE
     rm $MWA_COEFF_ARCHIVE
