@@ -6,6 +6,8 @@
 
 #include <aocommon/uvector.h>
 
+namespace wsclean {
+
 class NumberList {
  public:
   static aocommon::UVector<int> ParseIntList(const std::string& str) {
@@ -41,4 +43,6 @@ class NumberList {
   }
 };
 
+}  // namespace wsclean
+
 #endif
