@@ -7,6 +7,8 @@
 
 #include <casacore/ms/MeasurementSets/MeasurementSet.h>
 
+namespace wsclean {
+
 struct ObservationInfo {
   double phaseCentreRA = 0.0;
   double phaseCentreDec = 0.0;
@@ -14,6 +16,14 @@ struct ObservationInfo {
   std::string observer;
   std::string fieldName;
 
+  /// Comparison operator, for use in tests.
+  bool operator==(const ObservationInfo& other) const {
+    return phaseCentreRA == other.phaseCentreRA &&
+           phaseCentreDec == other.phaseCentreDec &&
+           telescopeName == other.telescopeName && observer == other.observer &&
+           fieldName == other.fieldName;
+  }
+
   void Serialize(aocommon::SerialOStream& stream) const;
   void Unserialize(aocommon::SerialIStream& stream);
 };
@@ -22,4 +32,6 @@ struct ObservationInfo {
 struct ObservationInfo ReadObservationInfo(casacore::MeasurementSet& ms,
                                            size_t fieldId);
 
+}  // namespace wsclean
+
 #endif
