@@ -8,6 +8,8 @@
 
 #include <unistd.h>
 
+namespace wsclean {
+
 class Application {
  public:
   static void Run(const std::string& commandLine) {
@@ -41,4 +43,6 @@ class Application {
   }
 };
 
+}  // namespace wsclean
+
 #endif
