@@ -13,6 +13,8 @@
 #include <errno.h>
 #include <fcntl.h>
 
+namespace wsclean {
+
 /**
  * A memory-mapped buffer that is mapped to a file on disk using mmap().
  */
@@ -92,4 +94,6 @@ class MappedFile {
   int file_descriptor_ = -1;
 };
 
+}  // namespace wsclean
+
 #endif
