@@ -1,9 +1,10 @@
-import pytest
+import os
 import shutil
 import sys
-import os
 from subprocess import check_call
 
+import pytest
+
 # Append current directory to system path in order to import testconfig
 sys.path.append(".")
 
@@ -12,8 +13,11 @@ import config_vars as tcf
 
 
 @pytest.fixture(scope="session", autouse=True)
-def prepare():
-    # Change to directory containing the data
+def prepare_workdir():
+    # Changes to directory containing the data.
+    # Optionally removes the entire working directory
+    # at the end of a test session. Use with care!
+    cwd = os.getcwd()
     os.makedirs(tcf.WORKING_DIR, exist_ok=True)
     os.chdir(tcf.WORKING_DIR)
     os.makedirs(tcf.RESULTS_DIR, exist_ok=True)
@@ -29,6 +33,15 @@ def prepare():
         )
         check_call(["tar", "xf", tcf.MWA_COEFF_ARCHIVE])
 
+    yield
+
+    if (
+        "CLEANUP_WSCLEAN_TESTS" in os.environ
+        and int(os.environ["CLEANUP_WSCLEAN_TESTS"]) == 1
+    ):
+        os.chdir(cwd)
+        shutil.rmtree(tcf.WORKING_DIR)
+
 
 @pytest.fixture(scope="class")
 def prepare_large_ms():
@@ -51,19 +64,12 @@ def prepare_mock_ms():
     os.makedirs(tcf.MWA_MOCK_MS, exist_ok=True)
 
     if not os.path.isfile(tcf.MWA_MOCK_ARCHIVE):
-        check_call(
-            [
-                "wget",
-                "-q",
-                os.path.join(
-                    tcf.EVERYBEAM_DATA_URL, "MWA-single-timeslot.tar.bz2"
-                ),
-                "-O",
-                tcf.MWA_MOCK_ARCHIVE,
-            ]
+        url = os.path.join(
+            tcf.EVERYBEAM_DATA_URL, "MWA-single-timeslot.tar.bz2"
         )
+        check_call(f"wget -q {url} -O {tcf.MWA_MOCK_ARCHIVE}".split())
         check_call(
-            f"tar -xf {tcf.MWA_MOCK_ARCHIVE}  -C {tcf.MWA_MOCK_MS} --strip-components=1".split()
+            f"tar -xf {tcf.MWA_MOCK_ARCHIVE} -C {tcf.MWA_MOCK_MS} --strip-components=1".split()
         )
 
     # From python 3.8 onwards, use copytree(..., dirs_exist_ok=True)
@@ -74,6 +80,13 @@ def prepare_mock_ms():
         shutil.copytree(tcf.MWA_MOCK_MS, tcf.MWA_MOCK_FACET)
 
 
+@pytest.fixture(scope="function")
+def tmp_mwa_mock_facet(tmp_path):
+    mwa_mock_facet_path = tmp_path / tcf.MWA_MOCK_FACET
+    shutil.copytree(tcf.MWA_MOCK_FACET, mwa_mock_facet_path)
+    return mwa_mock_facet_path
+
+
 @pytest.fixture(scope="class")
 def prepare_model_image():
     if not os.path.isfile(tcf.MODEL_IMAGE):
@@ -81,18 +94,8 @@ def prepare_model_image():
         check_call(wget.split())
 
 
-@pytest.fixture(scope="session", autouse=True)
-def remove_workdir(request):
-    """
-    Fixture runs only at the end of a test session, and (NOTE!) removes
-    the entire working directory. Use with care!
-    """
-
-    def collect_cleanup():
-        if (
-            "CLEANUP_WSCLEAN_TESTS" in os.environ
-            and int(os.environ["CLEANUP_WSCLEAN_TESTS"]) == 1
-        ):
-            shutil.rmtree(tcf.WORKING_DIR)
-
-    request.addfinalizer(collect_cleanup)
+@pytest.fixture(scope="class")
+def prepare_mock_soltab():
+    if not os.path.isfile(tcf.MOCK_SOLTAB_2POL):
+        wget = f"wget -q {os.path.join(tcf.WSCLEAN_DATA_URL, tcf.MOCK_SOLTAB_2POL)}"
+        check_call(wget.split())
