@@ -1,11 +1,13 @@
-from wsgiref import validate
-import pytest
-import os, glob
+import glob
+import os
 import sys
-from astropy.io import fits
-import numpy as np
-from utils import validate_call, compute_rms
+from wsgiref import validate
 
+import h5py
+import numpy as np
+import pytest
+from astropy.io import fits
+from utils import assert_taql, compute_rms, validate_call
 
 # Append current directory to system path in order to import testconfig
 sys.path.append(".")
@@ -26,6 +28,79 @@ def name(name: str):
     return os.path.join(tcf.RESULTS_DIR, name)
 
 
+"""
+Checks if a specified pixel in a fits file is within 0.03 units
+of the given expected value.
+"""
+
+
+def check_image_pixel(position, expected_value, filename):
+    with fits.open(filename) as image:
+        value = image[0].data[position]
+    assert expected_value - 0.03 < value < expected_value + 0.03
+
+
+def set_test_gains_in_solution_file(solution_file):
+    with h5py.File(solution_file, "a") as table:
+        solset = table["sol000"]
+        # times, freq, ant, dir, pol
+        for i in range(0, 5):
+            if solset["amplitude000/val"].ndim == 5:
+                solset["amplitude000/val"][:, :, :, i, :] = i + 2
+                solset["phase000/val"][:, :, :, i, :] = 0
+            else:
+                solset["amplitude000/val"][:, :, :, i] = i + 2
+                solset["phase000/val"][:, :, :, i] = 0
+        solset["amplitude000/weight"][:] = 1
+        solset["phase000/weight"][:] = 1
+
+
+@pytest.fixture
+def model_file_fixture():
+    model_3c196 = """Format = Name, Patch, Type, Ra, Dec, I, Q, U, V, SpectralIndex, LogarithmicSI, ReferenceFrequency='150.e6', MajorAxis, MinorAxis, Orientation
+,A,POINT, 08:13:36.0, 48.13.03.000,
+,B,POINT, 08:23:36.0, 48.13.03.000,
+,C,POINT, 08:03:36.0, 48.13.03.000,
+,D,POINT, 08:13:36.0, 49.45.00.000,
+,E,POINT, 08:13:36.0, 47.15.00.000,
+3c196, A, POINT, 08:13:36.0, 48.13.03.000, 0, 1, 0, 0, [0.0], false, , , ,
+left, B, POINT, 08:23:36.0, 48.13.03.000, 0, 0, 1, 0, [0.0], false, , , ,
+right, C, POINT, 08:03:36.0, 48.13.03.000, 0, 0, 0, 1, [0.0], false, , , ,
+top, D, POINT, 08:13:36.0, 49.45.00.000, 1, 0, 0, 0, [0.0], false, , , ,
+bottom, E, POINT, 08:13:36.0, 47.15.00.000, 0, -1, 0, 0, [0.0], false, , , ,
+"""
+    with open("testmodel.txt", "w") as f:
+        f.write(model_3c196)
+
+
+@pytest.fixture
+def region_file_fixture():
+    # Created using:
+    # ds9_facet_generator.py --h5 out.h5 --ms LOFAR_3C196.ms/ --imsize 2500 --pixelscale 600 --outputfile 3c196-with-5-facets.reg
+    # The h5 parm can be created with Dp3, e.g.:
+    # DP3 msin=3c196-simulation.ms/ msout=test.ms msout.overwrite=True steps=[ddecal] ddecal.sourcedb=testmodel.txt ddecal.h5parm=out.h5 ddecal.solint=100 ddecal.mode=scalar ddecal.solveralgorithm=directioniterative
+    facets_3c196 = """# Region file format: DS9 version 4.1
+global color=green dashlist=8 3 width=1 font="helvetica 10 normal roman" select=1
+fk5
+
+polygon(124.65026,47.72693,124.65027,48.97713,122.14973,48.97713,122.14974,47.72693)
+polygon(170.51499,-18.67893,242.70949,37.18997,245.26832,39.29522,124.65027,48.97713,124.65026,47.72693,156.25106,-22.65241)
+polygon(4.05193,37.22353,76.33129,-18.69386,90.54899,-22.65230,122.14974,47.72693,122.14973,48.97713,1.53191,39.29548)
+polygon(1.53191,39.29548,122.14973,48.97713,124.65027,48.97713,245.26832,39.29522)
+polygon(156.25106,-22.65241,124.65026,47.72693,122.14974,47.72693,90.54899,-22.65230)
+"""
+    with open("3c196-with-5-facets.reg", "w") as f:
+        f.write(facets_3c196)
+
+
+# Dimensions are pol, freq, y, x
+i_source_pos = (0, 0, 1802, 1250)
+q_source_pos = (0, 0, 1250, 1250)
+negative_q_source_pos = (0, 0, 902, 1250)
+u_source_pos = (0, 0, 1260, 651)
+v_source_pos = (0, 0, 1260, 1850)
+
+
 @pytest.mark.usefixtures("prepare_large_ms")
 class TestLongSystem:
     """
@@ -138,11 +213,12 @@ class TestLongSystem:
         validate_call(s.split())
 
     @pytest.mark.parametrize(
-        "gridder, test_name", (["", "shift-ws"], ["-use-wgridder", "shift-wg"])
+        "gridder, test_name",
+        (["wstacking", "shift-ws"], ["wgridder", "shift-wg"]),
     )
     def test_shift_image(self, gridder, test_name):
         # Shift the image with w-stacking and w-gridder gridder
-        s = f"{tcf.WSCLEAN} {gridder} -name {name(test_name)} -mgain 0.8 -auto-threshold 5 -niter 1000000 -make-psf {tcf.DIMS_RECTANGULAR} -shift 08h09m20s -39d06m54s -no-update-model-required {tcf.MWA_MS}"
+        s = f"{tcf.WSCLEAN} -gridder {gridder} -name {name(test_name)} -mgain 0.8 -auto-threshold 5 -niter 1000000 -make-psf {tcf.DIMS_RECTANGULAR} -shift 08h09m20s -39d06m54s -no-update-model-required {tcf.MWA_MS}"
         validate_call(s.split())
 
     def test_shifted_source_list(self):
@@ -165,7 +241,7 @@ class TestLongSystem:
     def test_missing_channels_in_deconvolution(self):
         # The test set has some missing MWA subbands. One MWA subband is 1/24 of the data (32/768 channels), so
         # by imaging with -channels-out 24, it is tested what happens when an output channel has no data.
-        s = f"{tcf.WSCLEAN} -name {name('missing-channels-in-deconvolution')} -use-wgridder {tcf.DIMS_LARGE} -baseline-averaging 2.0 -no-update-model-required -niter 150000 -auto-threshold 2.0 -auto-mask 5.0 -mgain 0.9 -channels-out 24 -join-channels -fit-spectral-pol 4 {tcf.MWA_MS}"
+        s = f"{tcf.WSCLEAN} -name {name('missing-channels-in-deconvolution')} -gridder wgridder {tcf.DIMS_LARGE} -baseline-averaging 2.0 -no-update-model-required -niter 150000 -auto-threshold 2.0 -auto-mask 5.0 -mgain 0.9 -channels-out 24 -join-channels -fit-spectral-pol 4 {tcf.MWA_MS}"
         validate_call(s.split())
 
     def test_grid_with_beam(self):
@@ -219,7 +295,7 @@ class TestLongSystem:
         validate_call(h5download.split())
 
         name = f"facet-h5-{npol}pol"
-        s = f"{tcf.WSCLEAN} -use-wgridder -name {name} -apply-facet-solutions mock_soltab_{npol}pol.h5 ampl000,phase000 -pol xx,yy -facet-regions {tcf.FACETFILE_4FACETS} {tcf.DIMS_LARGE} -join-polarizations -interval 10 14 -niter 1000000 -auto-threshold 5 -mgain 0.8 {tcf.MWA_MS}"
+        s = f"{tcf.WSCLEAN} -gridder wgridder -name {name} -apply-facet-solutions mock_soltab_{npol}pol.h5 ampl000,phase000 -pol xx,yy -facet-regions {tcf.FACETFILE_4FACETS} {tcf.DIMS_LARGE} -join-polarizations -interval 10 14 -niter 1000000 -auto-threshold 5 -mgain 0.8 {tcf.MWA_MS}"
         validate_call(s.split())
 
         # Check for output images
@@ -240,7 +316,7 @@ class TestLongSystem:
 
     def test_facet_beam(self):
         # Test facet beam, using 4 polarizations
-        s = f"{tcf.WSCLEAN} -name {name('nfacets-iquv-facet-beam')} -interval 10 14 -apply-facet-beam -pol iquv \
+        s = f"{tcf.WSCLEAN} -name {name('nfacets-iquv-facet-beam')} -interval 10 14 -apply-facet-beam -pol iquv -join-polarizations \
             -facet-regions {tcf.FACETFILE_NFACETS} {tcf.DIMS_RECTANGULAR} \
                 -mwa-path . {tcf.MWA_MS}"
         validate_call(s.split())
@@ -424,7 +500,7 @@ class TestLongSystem:
         validate_call(s.split())
 
         # Generate dirty image and PSF_GRID_SIZE_1D x PSF_GRID_SIZE_1D direction-dependent PSFs
-        s = f"{tcf.WSCLEAN} -name {name('DD-PSFs')} -data-column MODEL_DATA -parallel-deconvolution 1600 -no-reorder -size 4800 4800 -scale 5asec -mgain 0.8 -niter 10000000 -threshold 10.0mJy -auto-mask 5.0 -weight briggs -1 -padding 1.2 -gridder idg -grid-with-beam -beam-mode array_factor -aterm-kernel-size 15 -beam-aterm-update 120 -dd-psf-grid 3 3 -nmiter 1 -python-deconvolution {deconvolution_script} {tcf.SKA_MS}"
+        s = f"{tcf.WSCLEAN} -name {name('DD-PSFs')} -data-column MODEL_DATA -parallel-deconvolution 1600 -no-reorder -size 4800 4800 -scale 5asec -mgain 0.8 -niter 10000000 -abs-threshold 10.0mJy -auto-mask 5.0 -weight briggs -1 -padding 1.2 -gridder idg -grid-with-beam -beam-mode array_factor -aterm-kernel-size 15 -beam-aterm-update 120 -dd-psf-grid 3 3 -nmiter 1 -python-deconvolution {deconvolution_script} {tcf.SKA_MS}"
         validate_call(s.split())
 
         # Check whether the restoring beam for dd-psf and regular imaging is the same
@@ -558,3 +634,247 @@ class TestLongSystem:
                 assert os.path.isfile(image_name)
                 image_name = prefix + "-MFS-" + postfix
                 assert os.path.isfile(image_name)
+
+    def test_iquv_facet_beam_corrections(
+        self, model_file_fixture, region_file_fixture
+    ):
+        # Dp3 is used to predict 5 sources with different IQUV values into the measurement set
+        dp3_run = f"DP3 msin={tcf.LOFAR_3C196_MS} msout=3c196-simulation.ms msout.overwrite=True steps=[predict] predict.sourcedb=testmodel.txt predict.usebeammodel=True"
+        validate_call(dp3_run.split())
+
+        # Run a I-only deconvolution with facets and beam
+        base_cmd = f"""{tcf.WSCLEAN} -name facet-iquv-corrections
+-parallel-gridding 4 -facet-regions 3c196-with-5-facets.reg -apply-facet-beam
+-size 2500 2500 -scale 10asec -taper-gaussian 1amin -niter 1000 -mgain 0.8
+-nmiter 1 -maxuvw-m 20000"""
+        cmd = base_cmd + " 3c196-simulation.ms"
+        validate_call(cmd.split())
+
+        check_image_pixel(
+            i_source_pos, 1.0, "facet-iquv-corrections-image-pb.fits"
+        )
+
+        # Check consistency of Stokes I predict
+        predict_base_cmd = f"""{tcf.WSCLEAN} -predict -name facet-iquv-corrections
+-parallel-gridding 4 -facet-regions 3c196-with-5-facets.reg -apply-facet-beam
+-maxuvw-m 20000 -model-column PREDICTED_DATA"""
+        predict_cmd = predict_base_cmd + " 3c196-simulation.ms"
+        validate_call(predict_cmd.split())
+
+        taql_cmd = f"select PREDICTED_DATA-MODEL_DATA FROM 3c196-simulation.ms WHERE sumsqr(UVW) < 20000*20000 && ANY(PREDICTED_DATA-MODEL_DATA > 1e-3)"
+        assert_taql(taql_cmd, 0)
+
+        # Run a full IQUV deconvolution
+        cmd = base_cmd + " -pol iquv -join-polarizations 3c196-simulation.ms"
+        validate_call(cmd.split())
+
+        check_image_pixel(
+            i_source_pos, 1.0, "facet-iquv-corrections-I-image-pb.fits"
+        )
+        check_image_pixel(
+            q_source_pos, 1.0, "facet-iquv-corrections-Q-image-pb.fits"
+        )
+        check_image_pixel(
+            negative_q_source_pos,
+            -1.0,
+            "facet-iquv-corrections-Q-image-pb.fits",
+        )
+        check_image_pixel(
+            u_source_pos, 1.0, "facet-iquv-corrections-U-image-pb.fits"
+        )
+        check_image_pixel(
+            v_source_pos, 1.0, "facet-iquv-corrections-V-image-pb.fits"
+        )
+
+        # Check consistency of IQUV predict
+        predict_cmd = predict_base_cmd + " -pol iquv 3c196-simulation.ms"
+        # TODO this is not working yet: issue with join-polarizations
+        # validate_call(predict_cmd.split())
+        # assert_taql(taql_cmd, 0)
+
+        # Run a XX,YY deconvolution
+        cmd = (
+            base_cmd
+            + " -pol xx,yy -join-polarizations -squared-channel-joining 3c196-simulation.ms"
+        )
+        validate_call(cmd.split())
+
+        check_image_pixel(
+            i_source_pos, 1.0, "facet-iquv-corrections-XX-image-pb.fits"
+        )
+        check_image_pixel(
+            i_source_pos, 1.0, "facet-iquv-corrections-YY-image-pb.fits"
+        )
+        check_image_pixel(
+            q_source_pos, 1.0, "facet-iquv-corrections-XX-image-pb.fits"
+        )
+        check_image_pixel(
+            q_source_pos, -1.0, "facet-iquv-corrections-YY-image-pb.fits"
+        )
+        check_image_pixel(
+            negative_q_source_pos,
+            -1.0,
+            "facet-iquv-corrections-XX-image-pb.fits",
+        )
+        check_image_pixel(
+            negative_q_source_pos,
+            1.0,
+            "facet-iquv-corrections-YY-image-pb.fits",
+        )
+
+        # Check consistency of XXYY predict
+        predict_cmd = (
+            predict_base_cmd
+            + " -pol xxyy -join-polarizations 3c196-simulation.ms"
+        )
+        # TODO this is not working yet: issue with join-polarizations
+        # validate_call(predict_cmd.split())
+        # assert_taql(taql_cmd, 0)
+
+    def test_facet_scalar_corrections(
+        self, model_file_fixture, region_file_fixture
+    ):
+        # Perform simple solve to get a hdf5 parm file
+        solution_file = "scalar_correction_solutions.h5"
+        dp3_run = f"DP3 msin={tcf.LOFAR_3C196_MS} msout= steps=[ddecal] ddecal.sourcedb=testmodel.txt ddecal.h5parm={solution_file} ddecal.solveralgorithm=directioniterative ddecal.mode=scalar ddecal.maxiter=1"
+        validate_call(dp3_run.split())
+
+        set_test_gains_in_solution_file(solution_file)
+
+        # Dp3 is used to predict 5 sources with different IQUV values into the measurement set
+        dp3_run = f"DP3 msin={tcf.LOFAR_3C196_MS} msout=3c196-simulation.ms msout.overwrite=True steps=[h5parmpredict] h5parmpredict.sourcedb=testmodel.txt h5parmpredict.applycal.parmdb={solution_file} h5parmpredict.applycal.correction=amplitude000"
+        validate_call(dp3_run.split())
+
+        base_cmd = f"""{tcf.WSCLEAN} -name facet-scalar-corrections
+-parallel-gridding 4 -facet-regions 3c196-with-5-facets.reg -size 2500 2500
+-apply-facet-solutions {solution_file} amplitude000,phase000
+-scale 10asec -taper-gaussian 1amin -niter 1000 -mgain 0.8
+-nmiter 1 -maxuvw-m 20000"""
+        cmd = base_cmd + " -scalar-visibilities 3c196-simulation.ms"
+        validate_call(cmd.split())
+
+        check_image_pixel(
+            i_source_pos, 1.0, "facet-scalar-corrections-image-pb.fits"
+        )
+
+        # These next calls check if a predict results in the same values as what the previous deconvolution run produced
+        predict_cmd = f"""{tcf.WSCLEAN} -predict -name facet-scalar-corrections
+-parallel-gridding 4 -facet-regions 3c196-with-5-facets.reg -size 2500 2500
+-apply-facet-solutions {solution_file} amplitude000,phase000
+-scale 10asec -maxuvw-m 20000 -model-column PREDICTED_DATA 3c196-simulation.ms"""
+        validate_call(predict_cmd.split())
+
+        taql_cmd = f"select PREDICTED_DATA-MODEL_DATA FROM 3c196-simulation.ms WHERE sumsqr(UVW) < 20000*20000 && ANY(PREDICTED_DATA-MODEL_DATA > 1e-3)"
+        assert_taql(taql_cmd, 0)
+
+    def test_iquv_facet_dual_corrections(
+        self, model_file_fixture, region_file_fixture
+    ):
+        # Perform simple solve to get a hdf5 parm file
+        solution_file = "dual_correction_solutions.h5"
+        dp3_run = f"DP3 msin={tcf.LOFAR_3C196_MS} msout= steps=[ddecal] ddecal.sourcedb=testmodel.txt ddecal.h5parm={solution_file} ddecal.solveralgorithm=directioniterative ddecal.maxiter=1"
+        validate_call(dp3_run.split())
+
+        set_test_gains_in_solution_file(solution_file)
+
+        # Dp3 is used to predict 5 sources with different IQUV values into the measurement set
+        dp3_run = f"DP3 msin={tcf.LOFAR_3C196_MS} msout=3c196-simulation.ms msout.overwrite=True steps=[h5parmpredict] h5parmpredict.sourcedb=testmodel.txt h5parmpredict.usebeammodel=True h5parmpredict.applycal.parmdb={solution_file} h5parmpredict.applycal.correction=amplitude000"
+        validate_call(dp3_run.split())
+
+        base_cmd = f"""{tcf.WSCLEAN} -name facet-dual-corrections
+-parallel-gridding 4 -facet-regions 3c196-with-5-facets.reg -apply-facet-beam
+-apply-facet-solutions {solution_file} amplitude000,phase000 -size 2500 2500
+-scale 10asec -taper-gaussian 1amin -niter 1000 -mgain 0.8 -nmiter 1
+-maxuvw-m 20000 -no-update-model-required"""
+        cmd = base_cmd + " 3c196-simulation.ms"
+        validate_call(cmd.split())
+
+        check_image_pixel(
+            i_source_pos, 1.0, "facet-dual-corrections-image-pb.fits"
+        )
+
+        cmd = base_cmd + " -pol iquv -join-polarizations 3c196-simulation.ms"
+        validate_call(cmd.split())
+
+        check_image_pixel(
+            i_source_pos, 1.0, "facet-dual-corrections-I-image-pb.fits"
+        )
+        check_image_pixel(
+            q_source_pos, 1.0, "facet-dual-corrections-Q-image-pb.fits"
+        )
+        check_image_pixel(
+            negative_q_source_pos,
+            -1.0,
+            "facet-dual-corrections-Q-image-pb.fits",
+        )
+        check_image_pixel(
+            u_source_pos, 1.0, "facet-dual-corrections-U-image-pb.fits"
+        )
+        check_image_pixel(
+            v_source_pos, 1.0, "facet-dual-corrections-V-image-pb.fits"
+        )
+
+        # Prepare for applying solutions to diagonal (XX,YY) vis. To do so, first
+        # apply the beam so that the element's projection effect are removed. For diagonal
+        # visibilities, we want to have as little power in xy,yx as possible, since it is 'lost'.
+        dp3_run = f"DP3 msin=3c196-simulation.ms msout= steps=[applybeam]"
+        validate_call(dp3_run.split())
+
+        cmd = base_cmd + " -diagonal-visibilities 3c196-simulation.ms"
+        validate_call(cmd.split())
+
+        check_image_pixel(
+            i_source_pos, 1.0, "facet-dual-corrections-image-pb.fits"
+        )
+
+    def test_full_jones_facet_corrections(
+        self, model_file_fixture, region_file_fixture
+    ):
+        # Perform simple solve to get a hdf5 parm file
+        solution_file = "full_jones_correction_solutions.h5"
+        dp3_run = f"DP3 msin={tcf.LOFAR_3C196_MS} msout= steps=[ddecal] ddecal.mode=fulljones ddecal.sourcedb=testmodel.txt ddecal.h5parm={solution_file} ddecal.solveralgorithm=directioniterative ddecal.maxiter=1"
+        validate_call(dp3_run.split())
+
+        with h5py.File(solution_file, "a") as table:
+            solset = table["sol000"]
+            n_directions = solset["amplitude000/val"].shape[3]
+            for i in range(0, n_directions):
+                # times, freq, ant, dir, pol
+                solset["amplitude000/val"][:, :, :, i, :] = [
+                    0,
+                    i + 2,
+                    i + 2,
+                    0,
+                ]
+                solset["phase000/val"][:, :, :, i, :] = [-0.1, 0.1, -0.2, 0.15]
+            solset["amplitude000/weight"][:] = 1
+            solset["phase000/weight"][:] = 1
+
+        dp3_run = f"DP3 msin={tcf.LOFAR_3C196_MS} msout=3c196-simulation.ms msout.overwrite=True steps=[h5parmpredict] h5parmpredict.sourcedb=testmodel.txt h5parmpredict.usebeammodel=True h5parmpredict.applycal.parmdb={solution_file} h5parmpredict.applycal.correction=fulljones h5parmpredict.applycal.soltab=[amplitude000,phase000]"
+        validate_call(dp3_run.split())
+
+        wsclean_run = f"""{tcf.WSCLEAN} -name full-jones-facet-corrections
+-parallel-gridding 4 -facet-regions 3c196-with-5-facets.reg -apply-facet-beam
+-apply-facet-solutions {solution_file} amplitude000,phase000 -size 2500 2500
+-scale 10asec -taper-gaussian 1amin -niter 1000 -mgain 0.8 -nmiter 1
+-maxuvw-m 20000 -no-update-model-required  -pol iquv -join-polarizations
+3c196-simulation.ms"""
+        validate_call(wsclean_run.split())
+
+        check_image_pixel(
+            i_source_pos, 1.0, "full-jones-facet-corrections-I-image-pb.fits"
+        )
+        check_image_pixel(
+            q_source_pos, 1.0, "full-jones-facet-corrections-Q-image-pb.fits"
+        )
+        check_image_pixel(
+            negative_q_source_pos,
+            -1.0,
+            "full-jones-facet-corrections-Q-image-pb.fits",
+        )
+        check_image_pixel(
+            u_source_pos, 1.0, "full-jones-facet-corrections-U-image-pb.fits"
+        )
+        check_image_pixel(
+            v_source_pos, 1.0, "full-jones-facet-corrections-V-image-pb.fits"
+        )
